@@ -24,6 +24,8 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <iostream>
 
@@ -141,7 +143,7 @@ void BLANK::afc()
 // adjust "frequency" iaw with afc processing
 }
 
-int BLANK::rx_process(double *buf, int len)
+int BLANK::rx_process(const double *buf, int len)
 {
 	complex z;
 	int i;
