@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include "combo.h"
 
 void popbrwsr_cb (Fl_Widget *v, long d);
@@ -76,7 +78,8 @@ void Fl_PopBrowser::pophide ()
 void Fl_PopBrowser::popbrwsr_cb_i (Fl_Widget *v, long d)
 {
   Fl_PopBrowser *me = (Fl_PopBrowser *)(v->parent());
-  Fl_Output *tgt = me->Rvals.Inp;
+//  Fl_Output *tgt = me->Rvals.Inp;
+  Fl_Input *tgt = me->Rvals.Inp;
 // update the return values
   if (tgt) {
     int row = (me->popbrwsr)->value();
@@ -101,7 +104,7 @@ void popbrwsr_cb (Fl_Widget *v, long d)
 }
 
 /* XPM Select_Browser Label */
-static char *brws_12[] = {
+static const char *brws_12[] = {
 /* width height num_colors chars_per_pixel */
 "    12    12        3            1",
 /* colors */
@@ -160,14 +163,15 @@ void btnComboBox_cb (Fl_Widget *v, void *d)
 }
 
 
-Fl_ComboBox::Fl_ComboBox (int X,int Y,int W,int H, char *L)
+Fl_ComboBox::Fl_ComboBox (int X,int Y,int W,int H, const char *L)
  : Fl_Group (X, Y, W, H, L)
 {
   width = W; height = H;
   Btn = new Fl_Button (X + W - 18, Y + 1, 18, H - 2);
   (new Fl_Pixmap (brws_12))->label (Btn);
   Btn->callback ((Fl_Callback *)btnComboBox_cb, 0);
-  Output = new Fl_Output (X, Y, W-18, H);
+//  Output = new Fl_Output (X, Y, W-18, H);
+  Output = new Fl_Input (X, Y, W-18, H);
   Brwsr = 0;
   datalist = new datambr *[FL_COMBO_LIST_INCR];
   maxsize = FL_COMBO_LIST_INCR;
@@ -185,6 +189,11 @@ void Fl_ComboBox::type (int t)
   listtype = t;
 }
 
+void Fl_ComboBox::readonly()
+{
+    Output->type(FL_NORMAL_OUTPUT);
+}
+
 // ComboBox value is contained in the Output widget
 
 void Fl_ComboBox::value( const char *s )
