@@ -27,6 +27,8 @@
 // ----------------------------------------------------------------------------
 
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -34,7 +36,6 @@
 #include <sys/time.h>
 
 #include "cw.h"
-#include "CWdialog.h"
 #include "misc.h"
 //#include "modeIO.h"
 #include "configuration.h"
@@ -44,6 +45,7 @@ void cw::tx_init(cSound *sc)
 	scard = sc;
 	phaseacc = 0;
 	lastsym = 0;
+	qrqphase = 0;
 }
 
 void cw::rx_init()
@@ -54,12 +56,17 @@ void cw::rx_init()
 	agc_peak = 0;	
 	digiscope->mode(Digiscope::SCOPE);
 	put_MODEstatus(mode);
+	usedefaultWPM = false;
 }
 
 void cw::init()
 {
 	modem::init();
-	set_cwXmtWPM(sldrCWxmtWPM->value());
+	trackingfilter->reset();
+	cw_adaptive_receive_threshold = (long int)trackingfilter->run(2 * cw_send_dot_length);
+	put_cwRcvWPM(cw_send_speed);
+	for (int i = 0; i < OUTBUFSIZE; i++)
+		outbuf[i] = qrqbuf[i] = 0.0;
 	rx_init();
 }
 
@@ -68,59 +75,64 @@ cw::~cw() {
 //		delete KeyLine;
 //		KeyLine = (modeIO *)0;
 //	}
+	if (cwfilter) delete cwfilter;
+	if (bitfilter) delete bitfilter;
+	if (trackingfilter) delete trackingfilter;
+//	if (keyshape) delete [] keyshape;
 }
 
 
-cw::cw() : morse(), modem()
+//cw::cw() : morse(), modem()
+cw::cw() : modem()
 {
 	double lp;
 
 	mode = MODE_CW;
 	freqlock = false;
-	cw_speed = 18;
-	frequency = 600;
-	tx_frequency = 600;
-	cw_bandwidth = 150;
+	frequency = progdefaults.CWsweetspot;
+	tx_frequency = get_txfreq_woffset();
+	risetime = progdefaults.CWrisetime;
+//	keyshape = new double[KNUM];
 	
 	samplerate = CWSampleRate;
 	fragmentsize = CWMaxSymLen;
-	bandwidth = cw_bandwidth;  // cw_bandwidth is selectable by the user
 
+	cw_speed  = progdefaults.CWspeed;
+	bandwidth = progdefaults.CWbandwidth;
+	
 	cw_send_speed = cw_speed;
 	cw_receive_speed = cw_speed;
-//	cw_noise_spike_threshold = INITIAL_NOISE_THRESHOLD;
 	cw_adaptive_receive_threshold = 2 * DOT_MAGIC / cw_speed;
 	cw_noise_spike_threshold = cw_adaptive_receive_threshold / 4;
+	cw_send_dot_length = DOT_MAGIC / cw_send_speed;
+	cw_send_dash_length = 3 * cw_send_dot_length;
+	symbollen = (int)(1.0 * samplerate * cw_send_dot_length / USECS_PER_SEC);
 	
 	memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
 
 // block of variables that get updated each time speed changes
-	cw_in_sync = false;	// Synchronization flag 
 	pipesize = 512;
 	cwTrack = true;
 	phaseacc = 0.0;
 	pipeptr = 0;
 	agc_peak = 1.0;
 
-	sync_parameters();
-
-// init the cw rx tracking arrays, sums and indexes to intial rx speed
-	cw_dt_dot_tracking_sum = AVERAGE_ARRAY_LENGTH * cw_send_dot_length;
-	cw_dt_dash_tracking_sum = AVERAGE_ARRAY_LENGTH * cw_send_dash_length;
-
-	for (int i = 0; i < AVERAGE_ARRAY_LENGTH; i++) {
-		cw_dot_tracking_array[i] = cw_send_dot_length;
-		cw_dash_tracking_array[i] = cw_send_dash_length;
-	}
-	cw_dt_dot_index = 0;
-	cw_dt_dash_index = 0;
-
-	lp = 0.5 * cw_bandwidth / samplerate;
+	lp = 0.5 * bandwidth / samplerate;
 	cwfilter = new C_FIR_filter();
 	cwfilter->init_lowpass (CW_FIRLEN, DEC_RATIO, lp);
 	
 	bitfilter = new Cmovavg(8);
-	
+	bitfilterlen = (int)(samplerate / frequency / 2);
+	bitfilterlen = bitfilterlen < 2 ? 2 : bitfilterlen;
+	bitfilter->setLength(bitfilterlen);
+
+	trackingfilter = new Cmovavg(TRACKING_FILTER_SIZE);
+
+	makeshape();
+	sync_parameters();
+	wf->Bandwidth ((int)bandwidth);
+	init();
+
 }
 
 // sync_parameters()
@@ -129,48 +141,107 @@ cw::cw() : morse(), modem()
  
 void cw::sync_parameters()
 {
-	// Do nothing if we are already synchronized with speed/gap.
-	if (cw_in_sync) 
-		return;
+	int lowerwpm, upperwpm, nusymbollen;
 
-// Send parameters:
-// Calculate the length of a Dot as 1200000 / speed in wpm,
-// and the length of a Dash as three Dot lengths.
-	if (cw_send_speed < CW_MIN_SPEED)
-		cw_send_speed = CW_MIN_SPEED;
-	if (cw_send_speed > CW_MAX_SPEED)
-		cw_send_speed = CW_MAX_SPEED;
-	cw_send_dot_length = DOT_MAGIC / cw_send_speed;
-	cw_send_dash_length = 3 * cw_send_dot_length;
-	symbollen = (int)(1.0 * samplerate * cw_send_dot_length / USECS_PER_SEC);
-
-// Receive parameters:
-	cw_receive_speed = DOT_MAGIC / (cw_adaptive_receive_threshold / 2);
-
-// receive routines track speeds, but we put hard limits
-// on the speeds here if necessary.
-// (dot/dash threshold is 2 dots timing)
-	if (cw_receive_speed < CW_MIN_SPEED)
-		cw_receive_speed = CW_MIN_SPEED;
-	if (cw_receive_speed > CW_MAX_SPEED)
-		cw_receive_speed = CW_MAX_SPEED;
-	cw_adaptive_receive_threshold = 2 * DOT_MAGIC / cw_receive_speed;
-
-// Calculate the basic receive dot and dash lengths.
-	
-	if (cwTrack)
-		cw_receive_dot_length = DOT_MAGIC / cw_receive_speed;
+	if (usedefaultWPM == false)
+		cw_send_dot_length = DOT_MAGIC / progdefaults.CWspeed;
 	else
-		cw_receive_dot_length = cw_send_dot_length;
+		cw_send_dot_length = DOT_MAGIC / progdefaults.defCWspeed;
+		
+	cw_send_dash_length = 3 * cw_send_dot_length;
+	nusymbollen = (int)(1.0 * samplerate * cw_send_dot_length / USECS_PER_SEC);
+
+	if (symbollen != nusymbollen || risetime != progdefaults.CWrisetime) {
+		risetime = progdefaults.CWrisetime;
+		symbollen = nusymbollen;
+		makeshape();
+	}
+	int len = (int)(samplerate / frequency / 2);
+	len = len < 2 ? 2 : len;
+	if (bitfilterlen != len) {
+		bitfilterlen = len;
+		bitfilter->setLength(bitfilterlen);
+	}
+	
+// check if user changed the tracking or the cw default speed
+	if ((cwTrack != progdefaults.CWtrack) ||
+		(cw_send_speed != progdefaults.CWspeed)) {
+		trackingfilter->reset();
+		cw_adaptive_receive_threshold = (long int)trackingfilter->run(2 * cw_send_dot_length);
+		put_cwRcvWPM(cw_send_speed);
+	}
+	cwTrack = progdefaults.CWtrack;
+	cw_send_speed = progdefaults.CWspeed;
+	
+// Receive parameters:
+	lowerwpm = cw_send_speed - progdefaults.CWrange;
+	upperwpm = cw_send_speed + progdefaults.CWrange;
+	if (lowerwpm < progdefaults.CWlowerlimit)
+		lowerwpm = progdefaults.CWlowerlimit;
+	if (upperwpm > progdefaults.CWupperlimit) 
+		upperwpm = progdefaults.CWupperlimit;
+	cw_lower_limit = 2 * DOT_MAGIC / upperwpm;
+	cw_upper_limit = 2 * DOT_MAGIC / lowerwpm;
+
+	if (cwTrack)
+		cw_receive_speed = DOT_MAGIC / (cw_adaptive_receive_threshold / 2);
+	else {
+		cw_receive_speed = cw_send_speed;
+		cw_adaptive_receive_threshold = 2 * cw_send_dot_length;
+	}
+		
+	cw_receive_dot_length = DOT_MAGIC / cw_receive_speed;
 
 	cw_receive_dash_length = 3 * cw_receive_dot_length;
-	cw_noise_spike_threshold = cw_receive_dot_length / 2;
 
-// Set the parameters in sync flag.
-	cw_in_sync = true;
+	cw_noise_spike_threshold = cw_receive_dot_length / 4;
+
 }
 
 
+//=======================================================================
+// cw_update_tracking()
+// This gets called everytime we have a dot dash sequence or a dash dot
+// sequence. Since we have semi validated tone durations, we can try and
+// track the cw speed by adjusting the cw_adaptive_receive_threshold variable.
+// This is done with moving average filters for both dot & dash.
+//=======================================================================
+
+void cw::update_tracking(int idot, int idash)
+{
+	int dot, dash;
+	if (idot > cw_lower_limit && idot < cw_upper_limit)
+		dot = idot;
+	else
+		dot = cw_send_dot_length;
+	if (idash > cw_lower_limit && idash < cw_upper_limit)
+		dash = idash;
+	else
+		dash = cw_send_dash_length;
+	
+	cw_adaptive_receive_threshold = (long int)trackingfilter->run((dash + dot) / 2);
+	sync_parameters();
+}
+
+//=======================================================================
+//update_syncscope()
+//Routine called to update the display on the sync scope display.
+//For CW this is an o scope pattern that shows the cw data stream.
+//=======================================================================
+
+void cw::update_Status()
+{
+	static char RXmsg[20];
+	static char TXmsg[20];
+	sprintf(RXmsg,"Rx %d", cw_receive_speed);
+	if (usedefaultWPM)
+		sprintf(TXmsg,"Tx %d **", progdefaults.defCWspeed);
+	else
+		sprintf(TXmsg,"Tx %d", progdefaults.CWspeed);
+	put_Status1(RXmsg);
+	put_Status2(TXmsg);	
+}
+
 //=======================================================================
 //update_syncscope()
 //Routine called to update the display on the sync scope display.
@@ -186,9 +257,9 @@ void cw::update_syncscope()
 		scopedata[i] = 0.1 + 0.8 * pipe[j] / agc_peak;
 	}
 	set_scope(scopedata, pipesize, false);
-
-	cwRcvWPM = cw_receive_speed;
-	put_cwRcvWPM(cwRcvWPM);
+	scopedata.next(); // change buffers
+	put_cwRcvWPM(cw_receive_speed);
+	update_Status();
 }
 
 
@@ -198,19 +269,21 @@ void cw::update_syncscope()
 // Called with a block (512 samples) of audio.
 //=======================================================================
 
-int cw::rx_process(double *buf, int len)
+int cw::rx_process(const double *buf, int len)
 {
 	complex z;
 	double delta;
 	double value;
-	char *c;
+	const char *c;
 
 // check if user changed filter bandwidth
-	if (bandwidth != cw_bandwidth) {
-		cw_bandwidth = (int)bandwidth;
+	if (bandwidth != progdefaults.CWbandwidth) {
+		bandwidth = progdefaults.CWbandwidth;
 		cwfilter->init_lowpass (CW_FIRLEN, DEC_RATIO, 0.5 * bandwidth / samplerate);
+		wf->Bandwidth ((int)bandwidth);
 	}
-	// compute phase increment expected at our specific rx tone freq 
+
+// compute phase increment expected at our specific rx tone freq 
 	delta = 2.0 * M_PI * frequency / samplerate;
 
 	while (len-- > 0) {
@@ -278,47 +351,6 @@ int cw::usec_diff(unsigned int earlier, unsigned int later)
 }
 
 
-//=======================================================================
-// cw_update_tracking()
-// This gets called everytime we have a dot dash sequence or a dash dot
-// sequence. Since we have semi validated tone durations, we can try and
-// track the cw speed by adjusting the cw_adaptive_receive_threshold variable.
-// This is done by running averages where the newest data replaces
-// the oldest data.
-//=======================================================================
-
-void cw::update_tracking(int dot, int dash)
-{
-	int average_dot;	// Averaged dot length 
-	int average_dash;	// Averaged dash length 
-
-// Update the dot data held for averaging.
-	if (dot) {
-	cw_dt_dot_tracking_sum -= cw_dot_tracking_array[cw_dt_dot_index];
-	cw_dot_tracking_array[cw_dt_dot_index++] = dot;
-	cw_dt_dot_index %= AVERAGE_ARRAY_LENGTH;
-	cw_dt_dot_tracking_sum += dot;
-	}
-// Update the dash data held for averaging.
-	if (dash) {
-	cw_dt_dash_tracking_sum -= cw_dash_tracking_array[cw_dt_dash_index];
-	cw_dash_tracking_array[cw_dt_dash_index++] = dash;
-	cw_dt_dash_index %= AVERAGE_ARRAY_LENGTH;
-	cw_dt_dash_tracking_sum += dash;
-	}
-// update speed values based on new dot/dash avgs
-// Recalculate the adaptive threshold from the values currently
-// held in the averaging arrays.  The threshold is calculated as
-// (avg dash length - avg dot length) / 2 + avg dot_length.
-	average_dot = cw_dt_dot_tracking_sum / AVERAGE_ARRAY_LENGTH;
-	average_dash = cw_dt_dash_tracking_sum / AVERAGE_ARRAY_LENGTH;
-	cw_adaptive_receive_threshold = (average_dash - average_dot) / 2 + average_dot;
-
-// force a recalc and limits checks on all internal timing variables
-	cw_in_sync = false;
-	sync_parameters();
-}
-
 //=======================================================================
 // handle_event()
 //    high level cw decoder... gets called with keyup, keydown, reset and
@@ -330,7 +362,7 @@ void cw::update_tracking(int dot, int dash)
 //    If there is no data ready, CW_ERROR is returned.
 //=======================================================================
 
-int cw::handle_event(int cw_event, char **c)
+int cw::handle_event(int cw_event, const char **c)
 {
 	static int space_sent = true;	// for word space logic
 	static int last_element = 0;	// length of last dot/dash
@@ -342,8 +374,7 @@ int cw::handle_event(int cw_event, char **c)
 		cw_receive_state = RS_IDLE;
 		cw_rr_current = 0;			// reset decoding pointer
 		smpl_ctr = 0;					// reset audio sample counter
-		memset(rx_rep_buf, 0,
-		       sizeof(rx_rep_buf));
+		memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
 		break;
 	case CW_KEYDOWN_EVENT:
 // A receive tone start can only happen while we
@@ -353,8 +384,7 @@ int cw::handle_event(int cw_event, char **c)
 // first tone in idle state reset audio sample counter
 		if (cw_receive_state == RS_IDLE) {
 			smpl_ctr = 0;
-			memset(rx_rep_buf, 0,
-			       sizeof(rx_rep_buf));
+			memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
 			cw_rr_current = 0;
 		}
 // save the timestamp
@@ -445,6 +475,7 @@ int cw::handle_event(int cw_event, char **c)
 			return CW_ERROR;
 // MEDIUM time since keyup... check for character space
 // one shot through this code via receive state logic
+// FARNSWOTH MOD HERE -->
 		if (element_usec >= (2 * cw_receive_dot_length) &&
 		    element_usec <= (4 * cw_receive_dot_length) &&
 		    cw_receive_state == RS_AFTER_TONE) {
@@ -459,6 +490,7 @@ int cw::handle_event(int cw_event, char **c)
 			return CW_SUCCESS;
 		}
 // LONG time since keyup... check for a word space
+// FARNSWOTH MOD HERE -->
 		if ((element_usec > (4 * cw_receive_dot_length)) && !space_sent) {
 			*c = " "; 
 			space_sent = true;
@@ -476,55 +508,21 @@ int cw::handle_event(int cw_event, char **c)
 // cw transmit routines
 // Define the amplitude envelop for key down events (32 samples long)      
 // this is 1/2 cycle of a raised cosine                                    
-// the tables with 32 entries give about 4ms rise and fall times           
-// when using 8000 samples/sec. This shaping of the cw pulses is           
-// very necssary to avoid having a very wide and clicky cw signal          
-// when using the sound card to gen cw. When using the rig key input       
-// the shaping is done in the rig hardware, but we want to be able to      
-// pick one cw signal out of a cluster and be able to respond on his freq. 
 //===========================================================================
 
-#define KNUM 32
-// keydown wave shape
-double kdshape[KNUM] = {
-	0.00240750255310301, 0.00960708477768751,
-	0.02152941088003600, 0.03805966253618680,
-	0.05903864465505320, 0.08426431851158830, 
-	0.11349374748686800, 0.14644543667658500,
-	0.18280204383628200, 0.22221343555548300, 
-	0.26430005922814900, 0.30865659834558700,
-	0.35485587590940700, 0.40245296837259500, 
-	0.45098949048925500, 0.49999800980765500,
-	0.54900654829266300, 0.59754312772456200, 
-	0.64514031509964400, 0.69133972425796200,
-	0.73569643038517400, 0.77778325487450100, 
-	0.81719487928327800, 0.85355174876454100,
-	0.88650372738152000, 0.91573347010241700, 
-	0.94095947900139100, 0.96193881423287900,
-	0.97846943367117300, 0.99039213868324900, 
-	0.99759210729604500, 0.99999999999295900
-};
-
-// keyup wave shape
-double kushape[KNUM] = {
-	0.99999999999295900, 0.99759210729604500, 
-	0.99039213868324900, 0.97846943367117300,
-	0.96193881423287900, 0.94095947900139100, 
-	0.91573347010241700, 0.88650372738152000,
-	0.85355174876454100, 0.81719487928327800, 
-	0.77778325487450100, 0.73569643038517400,
-	0.69133972425796200, 0.64514031509964400, 
-	0.59754312772456200, 0.54900654829266300,
-	0.49999800980765500, 0.45098949048925500, 
-	0.40245296837259500, 0.35485587590940700,
-	0.30865659834558700, 0.26430005922814900, 
-	0.22221343555548300, 0.18280204383628200,
-	0.14644543667658500, 0.11349374748686800, 
-	0.08426431851158830, 0.05903864465505320,
-	0.03805966253618680, 0.02152941088003600, 
-	0.00960708477768751, 0.00240750255310301
-};
+double keyshape[KNUM];
 
+void cw::makeshape()
+{
+	for (int i = 0; i < KNUM; i++) keyshape[i] = 1.0;
+	knum = (int)(8 * risetime);
+	if (knum > symbollen)
+		knum = symbollen;
+	if (knum > KNUM) 
+		knum = KNUM;
+	for (int i = 0; i < knum; i++)
+		keyshape[i] = 0.5 * (1.0 - cos (M_PI * i / knum));
+}
 
 inline double cw::nco(double freq)
 {
@@ -533,7 +531,17 @@ inline double cw::nco(double freq)
 	if (phaseacc > M_PI)
 		phaseacc -= 2.0 * M_PI;
 
-	return cos(phaseacc);
+	return sin(phaseacc);
+}
+
+inline double cw::qrqnco()
+{
+	qrqphase += 2.0 * M_PI * 1600.0 / samplerate;
+
+	if (qrqphase > M_PI)
+		qrqphase -= 2.0 * M_PI;
+
+	return sin(qrqphase);
 }
 
 //=====================================================================
@@ -541,85 +549,127 @@ inline double cw::nco(double freq)
 // Sends a part of a morse character (one dot duration) of either
 // sound at the correct freq or silence. Rise and fall time is controlled
 // with a raised cosine shape.
+//
+// Left channel contains the shaped A2 CW waveform
+// Right channel contains a square wave burst of 1600 Hz that is used
+// to trigger a QRQ switch.  Right channel has pre and post timings for
+// proper switching of the QRQ switch before and after the A2 element.
+// If the Pre + Post timing exceeds the interelement spacing then the
+// Pre and / or Post is only applied at the beginning and end of the
+// character.
 //=======================================================================
 
-
-void cw::send_symbol(int symbol)
+void cw::send_symbol(int bits)
 {
 	double freq;
 	int sample = 0, i;
-	int currsym = symbol & 1;
-	int nextsym = (symbol >> 1) & 1;
-//	int symlen100 = (int) (samplerate * 0.012);
 	int delta = 0;
 	int keydown;
 	int keyup;
+	int kpre;
+	int kpost;
 	int duration = 0;
+	int symlen = 0;
+	double dsymlen = 0.0;
+	int currsym = bits & 1;
+	
+	freq = get_txfreq_woffset();
 
-	freq = tx_frequency;
+    if ((currsym == 1) && (lastsym == 0)) {
+    	phaseacc = 0.0;
+//    	qrqphase = 0.0;
+    }
 
 	if ((currsym == 1 && lastsym == 0) || (currsym == 0 && lastsym == 1))
 		delta = (int) (symbollen * (progdefaults.CWweight - 50) / 100.0);
-	keydown = symbollen - 2 * KNUM + delta;
-	keyup = symbollen - delta;
 
 	if (currsym == 1) {
-		if (cw_send_speed <= 100) {
-			for (i = 0; i < KNUM; i++, sample++) {
-				if (lastsym == 0)
-					outbuf[sample] = nco(freq) * kdshape[i];
-				else
-					outbuf[sample] = nco(freq);
-			}
-			for (i = 0; i < keydown; i++, sample++) {
+		dsymlen = symbollen * 4.0 / (progdefaults.CWdash2dot + 1.0);
+		if (lastsym == 1 && currsym == 1)
+			dsymlen *= ((progdefaults.CWdash2dot - 1.0) / 2.0);
+		symlen = (int) dsymlen;
+		if (symlen < knum) symlen = knum;
+	} else
+		symlen = symbollen;
+//	cout << (symbol & 1) << "-" << symlen << ", ";
+
+	if (delta < -(symlen - knum)) delta = -(symlen - knum);
+	if (delta > (symlen - knum)) delta = symlen - knum;
+
+	keydown = symlen - knum + delta;
+	keyup = symlen - knum - delta;
+	
+	kpre = (int)(progdefaults.CWpre * 8);
+	kpost = (int)(progdefaults.CWpost * 8);
+	
+	if ( (kpre + kpost) > keyup) {
+		kpre = keyup / 2;
+		kpost = keyup - kpre;
+	}
+	if (bits == 4) {
+		kpost = (int)(progdefaults.CWpost * 8);
+		if (kpost > keyup)
+			kpost = keyup;
+	}
+	if (firstelement) {
+		kpre = (int)(progdefaults.CWpre * 8);
+		kpost = 0;
+		if (kpre > keydown)
+			kpre = keydown;
+	}
+	
+	if (currsym == 1) { // keydown
+		for (i = 0; i < knum; i++, sample++) {
+			if (lastsym == 0)
+				outbuf[sample] = nco(freq) * keyshape[i];
+			else
 				outbuf[sample] = nco(freq);
-			}
-			for (i = 0; i < KNUM; i++, sample++ ) {
-				if (nextsym == 0)
-					outbuf[sample] = nco(freq) * kushape[i];
-				else
-					outbuf[sample] = nco(freq);
-			}
-		} else {
-			for (i = 0; i < KNUM; i += 2, sample++) {
-				if (lastsym == 0)
-					outbuf[sample] = nco(freq) * kdshape[i];
-				else
-					outbuf[sample] = nco(freq);
-			}
-			duration += KNUM / 2;
-			for (i = 0; i < keydown + KNUM; i++, sample++) {
-				outbuf[sample] = nco(freq);
-			}
-			for (i = 0; i < KNUM; i += 2, sample++ ) {
-				if (nextsym == 0)
-					outbuf[sample] = nco(freq) * kushape[i];
-				else
-					outbuf[sample] = nco(freq);
+			qrqbuf[sample] = qrqnco();
+		}
+		for (i = 0; i < keydown; i++, sample++) {
+			outbuf[sample] = nco(freq);
+			qrqbuf[sample] = qrqnco();
+		}
+		duration = knum + keydown;
+	}
+	else { // keyup
+		for (i = knum - 1; i >= 0; i--, sample++) {
+			if (lastsym == 1) {
+				outbuf[sample] = nco(freq) * keyshape[i];
+			} else {
+				outbuf[sample] = 0.0;
 			}
 		}
-		duration = keydown + 2 * KNUM;
-	} else {
-		if (lastsym == 1) {
-			for (i = 0; i < keyup; i++)
-				outbuf[i] = 0.0;
-			duration = keyup;
-		} else {
-			for (i = 0; i < symbollen; i++)
-				outbuf[i] = 0.0;
-			duration = symbollen;
+		for (i = 0; i < keyup; i++, sample++) {
+			outbuf[sample] = 0.0;
 		}
+
+		sample -= (knum + keyup);
+				
+		for (i = 0; i < knum + kpost ; i++, sample++) {
+			if (bits > 2 && lastsym == 1)
+				qrqbuf[sample] = qrqnco();
+			else
+				qrqbuf[sample] = 0.0;
+		}
+		for (i = 0; i < (keyup - kpre - kpost); i++, sample++)
+			qrqbuf[sample] = 0.0;
+		for (i = 0; i < kpre; i++, sample++)
+			if (bits > 4)
+				qrqbuf[sample] = qrqnco();
+			else
+				qrqbuf[sample] = 0.0;
+				
+		duration = knum + keyup;
 	}
 
-//	if (progdefaults.useCWkeylineRTS || progdefaults.useCWkeylineDTR) {
-//		if (currsym != lastsym)
-//			cw_keyline(currsym);
-//		std::cout << currsym; fflush(stdout);
-//		}
-
-	ModulateXmtr(outbuf, duration);
+	if (progdefaults.QSK)
+		ModulateStereo(outbuf, qrqbuf, duration);
+	else
+		ModulateXmtr(outbuf, duration);
 	
 	lastsym = currsym;
+	firstelement = false;
 }
 
 //=====================================================================
@@ -631,15 +681,12 @@ void cw::send_ch(int ch)
 {
 	int code;
 
-// compute number of samples of audio in one dot at current wpm
-	if (cw_send_speed != (int)cwXmtWPM) {
-		cw_send_speed = (int)cwXmtWPM;
-		cw_in_sync = false;
-	}
 	sync_parameters();
 // handle word space separately (7 dots spacing) 
-// last char already had 2 dots of inter-character spacing sent with it 
+// last char already had 2 elements of inter-character spacing 
+
 	if ((ch == ' ') || (ch == '\n')) {
+		firstelement = false;
 		send_symbol(0);
 		send_symbol(0);
 		send_symbol(0);
@@ -650,22 +697,28 @@ void cw::send_ch(int ch)
 	}
 
 // convert character code to a morse representation 
-	if ((ch < 256) && (ch >= 0))
+	if ((ch < 256) && (ch >= 0)) {
 		code = tx_lookup(ch); //cw_tx_lookup(ch);
-	else
-		code = 0x4; // two dot spaces
+		firstelement = true;
+	} else {
+		code = 0x04; 	// two extra dot spaces
+		firstelement = false;
+	}
+
 // loop sending out binary bits of cw character 
 	while (code > 1) {
 		send_symbol(code);// & 1);
 		code = code >> 1;
+        FL_AWAKE();
 	}
+
 	if (ch != 0)
 		put_echo_char(ch);
 }
 
 //=====================================================================
 // cw_txprocess()
-// Read charcters from screen and send them out the sound card.
+// Read characters from screen and send them out the sound card.
 // This is called repeatedly from a thread during tx.
 //=======================================================================
 
@@ -678,37 +731,33 @@ int cw::tx_process()
 		stopflag = false;
 			return -1;
 	}
-	if (c != 0)
-		send_ch(c);
-	else
-		send_symbol(0);
-
+	send_ch(c);
 	return 0;
 }
 
-/*
-void cw::cw_keyup()
+void	cw::incWPM()
 {
-	if (progdefaults.useCWkeylineRTS)
-		KeyLine->clearRTS();
-	else if (progdefaults.useCWkeylineDTR)
-		KeyLine->clearDTR();
+	if (usedefaultWPM) return;
+	if (progdefaults.CWspeed < progdefaults.CWupperlimit) {
+		progdefaults.CWspeed++;
+		set_CWwpm();
+		update_Status();
+	}
 }
 
-void cw::cw_keydown()
+void	cw::decWPM()
 {
-	if (progdefaults.useCWkeylineRTS)
-		KeyLine->setRTS();
-	else if (progdefaults.useCWkeylineDTR)
-		KeyLine->setDTR();
+	if (usedefaultWPM) return;
+	if (progdefaults.CWspeed > progdefaults.CWlowerlimit) {
+		progdefaults.CWspeed--;
+		set_CWwpm();
+		update_Status();
+	}
 }
 
-void cw::cw_keyline(int symbol)
+void	cw::toggleWPM()
 {
-	if (symbol)
-		cw_keydown();
-	else
-		cw_keyup();
+	usedefaultWPM = !usedefaultWPM;
+	update_Status();
 }
-*/
 
