@@ -22,11 +22,12 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
 #include <stdlib.h>
 
 #include <iostream>
 
-#include "version.h"
 #include "Config.h"
 
 #include "dominoex.h"
@@ -49,12 +50,7 @@ void dominoex::tx_init(cSound *sc)
 	txprevtone = 0;
 	counter = 0;
 	phaseacc = 0.0;
-	if (trx_state != STATE_TUNE && progdefaults.sendid == true)
-		wfid->transmit(mode);
-	else if (trx_state != STATE_TUNE && progdefaults.macroid == true) {
-		wfid->transmit(mode);
-		progdefaults.macroid = false;
-	}
+	videoText();
 }
 
 void dominoex::rx_init()
@@ -66,7 +62,6 @@ void dominoex::rx_init()
 	counter = 0;
 	phaseacc = 0.0;
 	freqerr = 0.0;
-	clear_StatusMessages();
 	put_MODEstatus(mode);
 	put_sec_char(0);
 }
@@ -89,7 +84,7 @@ void dominoex::restart()
 
 	strSecXmtText = txtSecondary->value();
 	if (strSecXmtText.length() == 0)
-		strSecXmtText = "fldigi "FLDIGI_VERSION" ";
+		strSecXmtText = "fldigi "PACKAGE_VERSION" ";
 }
 
 void dominoex::init()
@@ -106,9 +101,8 @@ dominoex::~dominoex()
 	if (binsfft) delete binsfft;
 	if (hilbert) delete hilbert;
 	if (pipe) delete [] pipe;
-	if (scopedata) delete [] scopedata;
 	if (filt) delete filt;
-	if (wfid) delete wfid;
+//	if (wfid) delete wfid;
 }
 
 dominoex::dominoex(trx_mode md)
@@ -182,7 +176,8 @@ dominoex::dominoex(trx_mode md)
 	afcfilt		= new Cmovavg(AFC_COUNT);
 
 	pipe = new domrxpipe[2 * symlen];
-	scopedata = new double[2 * symlen];
+	scopedata.alloc(2 * symlen);
+	videodata.alloc(numtones * 6);
 	pipeptr = 0;
 	symcounter = 0;
 	metric = 0.0;
@@ -201,7 +196,7 @@ dominoex::dominoex(trx_mode md)
 	fragmentsize = symlen;
 
 	s2n = 0.0;
-	wfid = new id(this);
+//	wfid = new id(this);
 
 	prev1symbol = prev2symbol = 0;
 	prev1vector = prev2vector = complex(0.0, 0.0);
@@ -292,7 +287,6 @@ int dominoex::harddecode(complex *in)
 void dominoex::update_syncscope(complex *bins)
 {
 	double max = 0, min = 1e6, range, mag;
-	double buffer[numtones * 6];
 	int numbins = numtones * 3 * (doublespaced ? 2 : 1);
 // dom waterfall
 	for (int i = 0; i < numbins; i++ ) {
@@ -308,10 +302,12 @@ void dominoex::update_syncscope(complex *bins)
 			if (mag < 0) mag = 0;
 		} else
 			mag = 0;
-		buffer[i] = 255*mag;
+		videodata[i] = 255*mag;
+	}
+	if (!squelchon || metric >= squelch) {
+		set_video(videodata, numbins);
+		videodata.next(); // change buffers
 	}
-	if (!squelchon || metric >= squelch)
-		set_video(buffer, numbins);
 
 // dom symbol synch data	
 	memset(scopedata, 0, 2 * symlen * sizeof(double));
@@ -321,7 +317,7 @@ void dominoex::update_syncscope(complex *bins)
 			scopedata[i] = (pipe[j].vector[prev1symbol]).mag();
 		}
 	set_scope(scopedata, 2 * symlen);
-
+	scopedata.next(); // change buffers
 }
 
 void dominoex::synchronize()
@@ -402,7 +398,7 @@ void dominoex::eval_s2n(complex curr, complex n)
 
 }
 
-int dominoex::rx_process(double *buf, int len)
+int dominoex::rx_process(const double *buf, int len)
 {
 	complex z, *bins, noise;
 
@@ -469,7 +465,7 @@ void dominoex::sendsymbol(int sym)
 	if (reverse)
 		tone = (numtones - 1) - tone;
 
-	f = tone * tonespacing + tx_frequency - bandwidth / 2;
+	f = tone * tonespacing + get_txfreq_woffset() - bandwidth / 2;
 	
 	phaseincr = twopi * f / samplerate;
 	
@@ -537,7 +533,7 @@ int dominoex::tx_process()
 		break;
 	case TX_STATE_DATA:
 		i = get_tx_char();
-		if (i == 0)
+		if (i == -1)
 			sendsecondary();
 		else if (i == 3)
 			txstate = TX_STATE_END;
@@ -554,6 +550,7 @@ int dominoex::tx_process()
 		break;
 	case TX_STATE_FLUSH:
 		flushtx();
+		cwid();
 		return -1;
 	}
 	return 0;
