@@ -34,6 +34,7 @@
 #include "filters.h"
 #include "fftfilt.h"
 #include "dominovar.h"
+#include "mbuffer.h"
 
 using namespace std;
 
@@ -52,8 +53,6 @@ public:
 		TX_STATE_FLUSH
 	};
 protected:
-// waterfall ID
-	id				*wfid;
 // common variables
 	double	phaseacc;
 	int		symlen;
@@ -71,7 +70,8 @@ protected:
 	domrxpipe			*pipe;
 	unsigned int	pipeptr;
 	unsigned int	datashreg;
-	double			*scopedata;
+	mbuffer<double, 0, 2>	scopedata;
+	mbuffer<double, 0, 2>	videodata;
 
 	complex currvector;
 	complex prev1vector;
@@ -123,7 +123,7 @@ public:
 	void	rx_init();
 	void	tx_init(cSound *sc);
 	void	restart();
-	int		rx_process(double *buf, int len);
+	int		rx_process(const double *buf, int len);
 	int		tx_process();
 };
 
