@@ -31,9 +31,9 @@
 #define _FELD_H
 
 #include "modem.h"
-#include "id.h"
 #include "filters.h"
 #include "fftfilt.h"
+#include "mbuffer.h"
 
 #define	FeldSampleRate	8000
 #define FeldMaxSymLen	1024
@@ -55,8 +55,6 @@
 class feld : public modem {
 enum FELD_STATE {PREAMBLE, POSTAMBLE, DATA};
 protected:
-// waterfall ID
-	id				*wfid;
 //rx
 	double rxphacc;
 	double rxcounter;
@@ -91,7 +89,7 @@ protected:
 	double OnShape[80];
 	double OffShape[80];
 	
-	int col_data[2*RxColumnLen];
+	mbuffer<int, 2*RxColumnLen> col_data;
 	int col_pointer;
 	int fntnbr;
 	
@@ -110,7 +108,7 @@ public:
 	void	rx_init();
 	void	tx_init(cSound *sc);
 	void 	restart() {};
-	int		rx_process(double *buf, int len);
+	int		rx_process(const double *buf, int len);
 	int		tx_process();
 	int		get_font_data(unsigned char c, int col);
 };
