@@ -31,15 +31,18 @@
 #include <FL/Fl_Menu_Bar.H>
 #include <FL/Fl_Pack.H>
 #include <FL/Fl_Text_Buffer.H>
-#include <FL/Fl_Text_Display.H>
-#include <FL/Fl_Text_Editor.H>
 #include <FL/Fl_Browser.H>
 #include <FL/Fl_Slider.H>
 #include <FL/Fl_Progress.H>
 #include <FL/Fl_Input.H>
-#include "combo.h"
+#include <FL/Fl_Tile.H>
+#include <FL/Fl_Light_Button.H>
+#include <FL/Fl_Check_Button.H>
 
+#include "combo.h"
 #include "TextView.h"
+#include "FTextView.h"
+#include "raster.h"
 #include "waterfall.h"
 #include "digiscope.h"
 #include "globals.h"
@@ -48,13 +51,16 @@
 extern Fl_Double_Window *fl_digi_main;
 extern cMixer			mixer;
 
-extern TextView			*ReceiveText;
-extern TextEdit			*TransmitText;
+extern ReceiveWidget		*ReceiveText;
+extern TransmitWidget		*TransmitText;
+extern Raster			*FHdisp;
+extern Fl_Tile_check		*TiledGroup;
 extern Fl_Text_Buffer	*rcvBuffer;
 extern Fl_Text_Buffer	*xmtBuffer;
 extern Fl_Box			*StatusBar;
 extern Fl_Box			*Status2;
 extern Fl_Box			*Status1;
+extern Fl_Box			*WARNstatus;
 extern Fl_Button		*MODEstatus;
 extern Fl_Slider		*sldrSquelch;
 extern Fl_Progress		*pgrsSquelch;
@@ -70,17 +76,20 @@ extern Fl_Input			*inpRstOut;
 extern Fl_Input			*inpQth;
 extern Fl_Input			*inpLoc;
 extern Fl_Input			*inpNotes;
+extern Fl_Input			*inpAZ;	// WA5ZNU
 extern Fl_Button		*qsoClear;
 extern Fl_Button		*qsoSave;
 extern Fl_Button		*btnMacroTimer;
 extern Fl_Slider		*valRcvMixer;
 extern Fl_Slider		*valXmtMixer;
+extern Fl_Light_Button	*afconoff;
+extern Fl_Light_Button	*sqlonoff;
+extern Fl_Check_Button	*chk_afconoff;
+extern Fl_Check_Button	*chk_sqlonoff;
 
+extern bool				useCheckButtons;
 extern bool				altMacros;
 
-//extern Fl_Browser *ReceiveText;
-//extern Fl_Browser *TransmitText;
-
 extern waterfall		*wf;
 extern Digiscope		*digiscope;
 
@@ -88,6 +97,7 @@ extern Digiscope		*digiscope;
 extern void create_fl_digi_main();
 extern Fl_Menu_Item menu_[];
 extern void activate_rig_menu_item(bool b);
+extern void activate_test_menu_item(bool b);
 
 extern void put_freq(double frequency);
 extern void put_Bandwidth(int bandwidth);
@@ -97,22 +107,27 @@ extern void set_scope(double *data, int len, bool autoscale = true);
 extern void set_phase(double phase, bool highlight);
 extern void set_rtty(double, double, double);
 extern void set_video(double *, int);
+extern void set_CWwpm();
 extern void put_rx_char(unsigned int data);
 extern void put_sec_char( char chr );
-extern void put_status(char *msg);
-extern void put_Status2(char *msg);
-extern void put_Status1(char *msg);
-extern void put_WARNstatus(bool);
+
+extern void put_status(const char *msg, double timeout = 0);
+extern void clear_status_cb(void *);
 extern void clear_StatusMessages();
 extern void put_MODEstatus(trx_mode mode);
+extern void put_Status1(const char *msg);
+extern void put_Status2(const char *msg);
+
+extern void put_WARNstatus(double);
+
 extern void put_rx_data(int *data, int len);
-extern char get_tx_char();
+extern int get_tx_char();
 extern int  get_secondary_char();
 extern void put_echo_char(unsigned int data);
 extern void resetRTTY();
 extern void resetOLIVIA();
 extern void resetDOMEX();
-extern void resetSoundCard(int);
+extern void resetSoundCard();
 extern void restoreFocus();
 extern void setReverse(int);
 extern void clearQSO();
@@ -123,34 +138,7 @@ extern void setSqlOnOff(bool b);
 extern bool QueryAfcOnOff();
 extern bool QuerySqlOnOff();
 
-
-extern void initCW();
-extern void initMFSK8();
-extern void initMFSK16();
-extern void initPSK31();
-extern void initPSK63();
-extern void initPSK125();
-extern void initQPSK31();
-extern void initQPSK63();
-extern void initQPSK125();
-extern void initRTTY();
-extern void initOLIVIA();
-extern void initDOMINOEX4();
-extern void initDOMINOEX5();
-extern void initDOMINOEX8();
-extern void initDOMINOEX11();
-extern void initDOMINOEX16();
-extern void initDOMINOEX22();
-extern void initFELDHELL();
-extern void initFSKHELL();
-extern void initFSKHELL105();
-extern void initTHROB1();
-extern void initTHROB2();
-extern void initTHROB4();
-extern void initTHROBX1();
-extern void initTHROBX2();
-extern void initTHROBX4();
-extern void initWWV();
-extern void initANALYSIS();
+extern void init_modem(trx_mode mode);
+extern void init_modem_sync(trx_mode mode);
 
 #endif
