@@ -26,41 +26,6 @@
 #ifndef _GLOBALS_H
 #define _GLOBALS_H
 
-extern char *mode_names[];
-extern char *state_names[];
-
-enum trx_mode {
-	MODE_MFSK16 = 0,
-	MODE_MFSK8,
-	MODE_OLIVIA,
-	MODE_RTTY,
-	MODE_THROB1,
-	MODE_THROB2,
-	MODE_THROB4,
-	MODE_THROBX1,
-	MODE_THROBX2,
-	MODE_THROBX4,
-	MODE_BPSK31,
-	MODE_QPSK31,
-	MODE_PSK63,
-	MODE_QPSK63,
-	MODE_PSK125,
-	MODE_QPSK125,
-	MODE_MT63,
-	MODE_FELDHELL,
-	MODE_FSKHELL,
-	MODE_FSKH105,
-	MODE_CW,
-    MODE_DOMINOEX4,
-	MODE_DOMINOEX5,
-    MODE_DOMINOEX8,
-    MODE_DOMINOEX11,
-    MODE_DOMINOEX16,
-    MODE_DOMINOEX22,
-	MODE_WWV,
-	MODE_ANALYSIS
-};
-
 enum state_t {
 	STATE_PAUSE = 0,
 	STATE_RX,
@@ -75,6 +40,66 @@ enum state_t {
 	STATE_IDLE,
 	STATE_NEW_MODEM
 };
+extern const char *state_names[];
+
+enum {
+	MODE_PREV = -2,
+	MODE_NEXT,
+
+	MODE_CW,
+
+	MODE_DOMINOEX4,
+	MODE_DOMINOEX5,
+	MODE_DOMINOEX8,
+	MODE_DOMINOEX11,
+	MODE_DOMINOEX16,
+	MODE_DOMINOEX22,
+
+	MODE_FELDHELL,
+	MODE_FSKHELL,
+	MODE_FSKH105,
+
+	MODE_MFSK8,
+	MODE_MFSK16,
+
+	MODE_BPSK31,
+	MODE_QPSK31,
+	MODE_PSK63,
+	MODE_QPSK63,
+	MODE_PSK125,
+	MODE_QPSK125,
+	MODE_PSK250,
+	MODE_QPSK250,
+
+	MODE_OLIVIA,
+
+	MODE_RTTY,
+
+	MODE_THROB1,
+	MODE_THROB2,
+	MODE_THROB4,
+	MODE_THROBX1,
+	MODE_THROBX2,
+	MODE_THROBX4,
+
+//	MODE_MT63,
+
+	MODE_WWV,
+	MODE_ANALYSIS,
+
+	NUM_MODES
+};
+typedef int trx_mode;
+
+struct mode_info_t {
+	trx_mode mode;
+	class modem **modem;
+	const char *sname;
+	const char *name;
+	const char *pskmail_name;
+};
+
+extern const struct mode_info_t mode_info[NUM_MODES];
 
 
 #endif
