@@ -10,8 +10,8 @@
 #include "interleave.h"
 #include "viterbi.h"
 #include "complex.h"
-#include "id.h"
 #include "mfskvaricode.h"
+#include "mbuffer.h"
 
 //#include "File_Selector.h"
 
@@ -65,8 +65,6 @@ enum {
 };
 
 protected:
-// waterfall ID
-	id				*wfid;
 // general
 	double phaseacc;
 	int symlen;
@@ -101,7 +99,7 @@ protected:
 
 	double met1;
 	double met2;
-	double *scopedata;
+	mbuffer<double, 0, 2> scopedata;
 	double s2n;
 	double sig;
 	double noise;
@@ -183,6 +181,8 @@ protected:
 	void	clearbits();
 	void	sendpic(unsigned char *data, int len);
 	bool	check_picture_header();
+	int	print_time_left(size_t bytes, char *str, size_t len,
+			  const char *prefix = "", const char *suffix = "");
 public:
 	mfsk (trx_mode md);
 	~mfsk ();
@@ -190,7 +190,7 @@ public:
 	void	rx_init();
 	void	tx_init(cSound *sc);
 	void	restart() {};
-	int		rx_process(double *buf, int len);
+	int		rx_process(const double *buf, int len);
 	int		tx_process();
 	void	shutdown();
 	void	makeTxViewer(int W, int H);
