@@ -3,23 +3,26 @@
 #ifndef	_MODEM_H
 #define	_MODEM_H
 
+#include <string>
+
 #include "threads.h"
 
 #include "misc.h"
-//#include "configuration.h"
 #include "sound.h"
 #include "waterfall.h"
-#include "ScopeDialog.h"
 #include "fl_digi.h"
-#include "CWdialog.h"
 #include "globals.h"
 #include "fl_digi.h"
 
-#include "config.h"
+#include "fldigi-config.h"
+#include "morse.h"
 
 #define	OUTBUFSIZE	16384
+// Constants for signal searching & s/n threshold
+#define SIGSEARCH 5
 
-class modem {
+
+class modem : public morse {
 protected:
 	trx_mode mode;
 	cSound	*scard;
@@ -71,7 +74,7 @@ public:
 	virtual void rx_init () = 0;
 	virtual void restart () = 0;
 	virtual int  tx_process () = 0;
-	virtual int  rx_process (double *, int len) = 0;
+	virtual int  rx_process (const double *, int len) = 0;
 	virtual void shutdown(){};
 	virtual void set1(int, int){};
 	virtual void set2(int, int){};
@@ -91,11 +94,10 @@ public:
 
 //	void 		set_mode(trx_mode);
 	trx_mode	get_mode();
-	char 		*get_mode_name() { return mode_names[get_mode()];}
+	const char	*get_mode_name() { return mode_info[get_mode()].sname;}
 	void		set_state(state_t);
 	void		set_state_wait(state_t);
 	state_t		get_state();
-	void		set_freq(int);
 	void		set_freq(double);
 	int			get_freq();
 	void		init_freqlock();
@@ -103,10 +105,9 @@ public:
 	void		set_sigsearch(int n) { sigsearch = n; freqerr = 0.0;};
 	bool		freqlocked();
 	double		get_txfreq();
+	double		get_txfreq_woffset();
 	void		set_metric(double);
 	double		get_metric();
-	void		set_txoffset(double);
-	double		get_txoffset();
 	void		set_reverse(bool on);
 	double		get_bandwidth();
 	void		set_bandwidth(double);
@@ -116,6 +117,9 @@ public:
 	int			get_echo_char();
 	
 	void		ModulateXmtr(double *, int);
+	void		ModulateStereo(double *, double *, int);
+	
+	void		videoText();
 	
 	void		set_stopflag(bool b) { stopflag = b;};
 
@@ -127,8 +131,47 @@ public:
 	double		get_cwXmtWPM();
 	void		set_cwXmtWPM(double);
 	double		get_cwRcvWPM();
-//	void		set_cwBandwidth(double);
-//	double		get_cwBandwidth();
+	virtual	void		incWPM() {};
+	virtual void		decWPM() {};
+	virtual void		toggleWPM() {};
+	
+// for waterfall id transmission
+public:
+
+private:
+	
+	static	int			wfid_mask[];
+	static  double		wfid_w[];
+	static  double		wfid_txpulse[];
+	static  double		wfid_outbuf[];
+	
+	void	wfid_make_pulse();
+	void	wfid_make_tones();
+	void	wfid_send(long int);
+	double	peakval(int symbol, int mask);
+	int 	findmask(int symbol);
+
+	void	wfid_sendchar(char c);
+	void	wfid_sendchars(string s);
+
+public:
+	void	wfid_text(string s);
+
+// for CW ID transmission
+private:
+	double	cwid_keyshape[128];
+	double	cwid_phaseacc;
+	int		RT;
+	int		cwid_symbollen;
+	int		cwid_lastsym;
+public:
+	void	cwid_makeshape();
+	double	cwid_nco(double freq);
+	void	cwid_send_symbol(int bits);
+	void	cwid_send_ch(int ch);
+	void	cwid_sendtext (string s);
+	void	cwid();
+
 };
 
 extern modem *cw_modem;
@@ -141,9 +184,11 @@ extern modem *feld_CMTmodem;
 extern modem *psk31_modem;
 extern modem *psk63_modem;
 extern modem *psk125_modem;
+extern modem *psk250_modem;
 extern modem *qpsk31_modem;
 extern modem *qpsk63_modem;
 extern modem *qpsk125_modem;
+extern modem *qpsk250_modem;
 extern modem *rtty_modem;
 extern modem *olivia_modem;
 extern modem *dominoex4_modem;
