@@ -31,15 +31,17 @@
 #include "filters.h"
 #include "pskcoeff.h"
 #include "pskvaricode.h"
-#include "id.h"
 
+//=====================================================================
 #define	PskSampleRate	(8000)
 #define PipeLen			(64)
 
+#define SNTHRESHOLD 2.0
+#define AFCDECAY 8
+//=====================================================================
+
 class psk : public modem {
 private:
-// waterfall ID
-	id				*wfid;
 // tx & rx
 	int				symbollen;
 	bool			_qpsk;
@@ -85,7 +87,9 @@ private:
 	void			tx_char(unsigned char c);
 	void			tx_flush();
 	void			update_syncscope();
-//	void			goertzel(complex);
+	void			signalquality();
+	void			findsignal();
+	void			phaseafc();
 	void			afc();
 	
 public:
@@ -95,7 +99,7 @@ public:
 	void rx_init();
 	void tx_init(cSound *sc);
 	void restart();
-	int rx_process(double *buf, int len);
+	int rx_process(const double *buf, int len);
 	int tx_process();
 	void searchDown();
 	void searchUp();
