@@ -10,12 +10,18 @@
 #include <sys/msg.h>
 #include <sys/shm.h>
 #include <sys/time.h>
+// for FreeBSD compilation
+#if (defined(__unix__) || defined(unix)) && !defined(USG)
+#include <sys/param.h>
+#endif
 #include <iostream>
 #include <string>
 #include <list>
 
 #include <hamlib/rig.h>
-#include <values.h>
+#ifndef BSD
+ #include <values.h>
+#endif
 #include <assert.h>
 #include <errno.h>
 
@@ -28,7 +34,7 @@ class RigException {
 public:
 char message[80];
 	RigException() { *message = 0;}
-	RigException (char * msg) {
+	RigException (const char * msg) {
 		memset(message,0,80);
 		strcpy(message, "Hamlib ");
 		strcat(message, msg);
