@@ -34,9 +34,12 @@
 #include "filters.h"
 #include "fftfilt.h"
 #include "digiscope.h"
-#include "id.h"
+//#include "id.h"
+#include "mbuffer.h"
+
+//#define	RTTY_SampleRate	8000
+#define RTTY_SampleRate 11025
 
-#define	RTTY_SampleRate	8000
 #define	RTTYMaxSymLen	1024
 
 #define	LETTERS	0x100
@@ -69,8 +72,6 @@ extern int _BITS[];
 
 class rtty : public modem {
 private:
-// waterfall ID
-	id				*wfid;
 
 	double shift;
 	int symbollen;
@@ -95,7 +96,7 @@ private:
 	Cmovavg *bitfilt;
 	fftfilt *bpfilt;
 
-	double pipe[RTTYMaxSymLen];
+	mbuffer<double, RTTYMaxSymLen, 2> pipe;
 	int pipeptr;
 
 	double bbfilter[RTTYMaxSymLen];
@@ -108,18 +109,19 @@ private:
 	int rxdata;
 	double posfreq, negfreq;
 	double poserr, negerr;
-	double avgratio;
 	int poscnt, negcnt;
 
 	double prevsymbol;
 	complex prevsmpl;
 
+	double FSKbuf[OUTBUFSIZE];		// signal array for qrq drive
+	double FSKphaseacc;
+	double FSKnco();
+
 	int rxmode;
 	int txmode;
 	int preamble;
 	
-	double sum, sum2, mean, stddev;
-	
 	void clear_syncscope();
 	void update_syncscope();
 	inline complex mixer(complex in);
@@ -144,7 +146,7 @@ public:
 	void rx_init();
 	void tx_init(cSound *sc);
 	void restart();
-	int rx_process(double *buf, int len);
+	int rx_process(const double *buf, int len);
 	int tx_process();
 
 	void searchDown();
