@@ -9,14 +9,26 @@
 #include "main.h"
 #include "globals.h"
 
-using namespace std;
-
 struct status {
-	trx_mode	lastmode;
+	int		lastmode;
+	int		mainX;
+	int		mainY;
+	int		mainW;
+	int		mainH;
+	int		RxTextHeight;
+	bool	rigShown;
+	int		rigX;
+	int		rigY;
+	int		carrier;
+	int		mag;
+	int		speed;
+	double	reflevel;
+	double	ampspan;
+	
 public:
-	void readLastState();
 	void saveModeState(trx_mode m);
 	void initLastState();
+	void saveLastState();
 	friend std::istream &operator>>(std::istream &stream, status &c);
 	friend std::ostream &operator<<(std::ostream &ostream, status c);
 };
