@@ -31,13 +31,9 @@
 #include "misc.h"
 #include "sound.h"
 #include "waterfall.h"
-#include "ScopeDialog.h"
-#include "CWdialog.h"
 #include "globals.h"
 #include "fl_digi.h"
 
-
-
 // ----------------------------------------------------------------------------
 
 extern	void	trx_start_modem(modem *);
@@ -46,6 +42,15 @@ extern	void	trx_close();
 extern	void	trx_reset(const char *scdev);
 extern	void	trx_start_macro_timer();
 
+extern	void	wait_modem_ready_prep(void);
+extern	void	wait_modem_ready_cmpl(void);
+extern	void	signal_modem_ready(void);
+
+extern Fl_Mutex		trx_mutex;
+extern Fl_Thread	trx_thread;
+extern state_t		trx_state;
+extern modem		*active_modem;
+
 extern	cSound 	*scard;
 
 #endif
