@@ -22,6 +22,8 @@
 //
 // ====================================================================
 
+#include <config.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ipc.h>
@@ -31,13 +33,12 @@
 
 #include "logger.h"
 #include "main.h"
-#include "version.h"
 #include "modem.h"
 
 #include <FL/fl_ask.H>
 
 struct FIELD {
-  char *name;
+  const char *name;
   unsigned int  size;
 };
   
@@ -78,9 +79,9 @@ FIELD fields[] = {
 #define ADIF_VERS "2.0"
 static string adif;
 
-char *ADIFHEADER = 
+const char *ADIFHEADER = 
 "<ADIF_VERS:%d>%s\n\
-<PROGRAMID:7>fldigi\n\
+<PROGRAMID:%d>%s\n\
 <PROGRAMVERSION:%d>%s\n\
 <EOH>\n\n";
 
@@ -97,7 +98,8 @@ int writeadif () {
 			return 1;
 		fprintf (adiFile, ADIFHEADER,
 			strlen(ADIF_VERS), ADIF_VERS,
-			strlen(FLDIGI_VERSION), FLDIGI_VERSION);
+			strlen(PACKAGE_NAME), PACKAGE_NAME,
+			strlen(PACKAGE_VERSION), PACKAGE_VERSION);
 		fclose(adiFile);
 	} else
 		fclose(adiFile);
@@ -148,15 +150,15 @@ int submit_log(void)
 		sprintf(adifdate,"%04d%02d%02d", tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
 		strftime(logtime, sizeof(logtime), "%H%M", tm);
 
-	string mode = mode_names[active_modem->get_mode()];
+	const char *mode = mode_info[active_modem->get_mode()].sname;
 	
 	sprintf(strFreqMhz,"%-12f", wf->dFreq()/1.0e6);
 
 	adif.erase();
 	
-	Fl::lock();
+	FL_LOCK();
 	log_msg = "";
-	log_msg = log_msg + "program:"	+ "fldigi v " + FLDIGI_VERSION + LOG_MSEPARATOR;
+	log_msg = log_msg + "program:"	+ PACKAGE_NAME + " v " + PACKAGE_VERSION + LOG_MSEPARATOR;
 	log_msg = log_msg + "version:"	+ LOG_MVERSION			+ LOG_MSEPARATOR;
 	log_msg = log_msg + "date:"		+ logdate				+ LOG_MSEPARATOR;
 	putadif(21, adifdate); 
@@ -169,7 +171,7 @@ int submit_log(void)
 	log_msg = log_msg + "mhz:"		+ strFreqMhz			+ LOG_MSEPARATOR;
 	putadif(12, strFreqMhz);
 	log_msg = log_msg + "mode:"		+ mode					+ LOG_MSEPARATOR;
-	putadif(14, mode.c_str());
+	putadif(14, mode);
 	log_msg = log_msg + "tx:"		+ inpRstOut->value()	+ LOG_MSEPARATOR;
 	putadif(24, inpRstOut->value());
 	log_msg = log_msg + "rx:"		+ inpRstIn->value() 	+ LOG_MSEPARATOR;
@@ -182,7 +184,7 @@ int submit_log(void)
 	putadif(13, inpLoc->value());
 	log_msg = log_msg + "notes:"	+ inpNotes->value()		+ LOG_MSEPARATOR;
 	putadif(16, inpNotes->value());
-	Fl::unlock();
+	FL_UNLOCK();
 
 	writeadif();
 	
