@@ -24,6 +24,8 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <iostream>
 
@@ -36,6 +38,8 @@
 
 #include "File_Selector.h"
 
+#include "qrunner.h"
+
 #undef  MAX
 #define MAX(a,b)		(((a)>(b))?(a):(b))
 #undef  CLAMP
@@ -45,6 +49,8 @@
 
 
 char mfskmsg[80];
+char txclr_tooltip[24];
+char txgry_tooltip[24];
 
 void  mfsk::tx_init(cSound *sc)
 {
@@ -52,12 +58,7 @@ void  mfsk::tx_init(cSound *sc)
 	txstate = TX_STATE_PREAMBLE;
 	bitstate = 0;
 	counter = 0;
-	if (trx_state != STATE_TUNE && progdefaults.sendid == true)
-		wfid->transmit(mode);
-	else if (trx_state != STATE_TUNE && progdefaults.macroid == true) {
-		wfid->transmit(mode);
-		progdefaults.macroid = false;
-	}
+	videoText();
 }
 
 void  mfsk::rx_init()
@@ -85,7 +86,6 @@ void mfsk::init()
 
 mfsk::~mfsk()
 {
-	if (scopedata) delete [] scopedata;
 	if (bpfilt) delete bpfilt;
 	if (rxinlv) delete rxinlv;
 	if (txinlv) delete txinlv;
@@ -95,7 +95,7 @@ mfsk::~mfsk()
 	if (pipe) delete [] pipe;
 	if (hbfilt) delete hbfilt;
 	if (binsfft) delete binsfft;
-	if (wfid) delete wfid;
+//	if (wfid) delete wfid;
 }
 
 mfsk::mfsk(trx_mode mfsk_mode) : modem()
@@ -149,12 +149,12 @@ mfsk::mfsk(trx_mode mfsk_mode) : modem()
 	bpfilt = new C_FIR_filter();
 	bpfilt->init_bandpass (127, 1, flo, fhi);
 
-	scopedata = new double [symlen * 2];
+	scopedata.alloc(symlen * 2);
 
 	samplerate = MFSKSampleRate;
 	fragmentsize = symlen;
 	bandwidth = (numtones - 1) * tonespacing;
-	wfid = new id(this);
+//	wfid = new id(this);
 	
 	picRxWin = 0;
 	picRxBox = 0;
@@ -239,7 +239,7 @@ void mfsk::recvpic(complex z)
 		
 		if (color) {
 			pixelnbr = rgb + row + 3*col;
-			updateRxPic( byte, pixelnbr);
+			QUEUE(CMP_CB(&mfsk::updateRxPic, this, byte, pixelnbr)); //updateRxPic( byte, pixelnbr);
 			if (++col == picW) {
 				col = 0;
 				if (++rgb == 3) {
@@ -248,11 +248,18 @@ void mfsk::recvpic(complex z)
 				}
 			}
 		} else {
-			updateRxPic( byte, pixelnbr++ );
-			updateRxPic( byte, pixelnbr++ );
-			updateRxPic( byte, pixelnbr++ );
-		}			
+			for (int i = 0; i < 3; i++)
+				QUEUE(CMP_CB(&mfsk::updateRxPic, this, byte, pixelnbr++)); //updateRxPic( byte, pixelnbr++ );
+		}
 		picf = 0.0;
+
+		int n = picW * picH * 3;
+		int s = snprintf(mfskmsg, sizeof(mfskmsg),
+				 "Recv picture: %04.1f%% done",
+				 (100.0f * pixelnbr) / n);
+		print_time_left(n - pixelnbr, mfskmsg + s,
+				sizeof(mfskmsg) - s, ", ", " left");
+		put_status(mfskmsg);
 	}
 }
 
@@ -434,7 +441,8 @@ void mfsk::update_syncscope()
 			scopedata[i] = (pipe[j].vector[prev1symbol]).mag();
 		}
 	set_scope(scopedata, 2 * symlen);
-	sprintf(mfskmsg, "s/n %3.0f dB", 20.0 * log10(s2n) );
+	scopedata.next(); // change buffers
+	snprintf(mfskmsg, sizeof(mfskmsg), "s/n %3.0f dB", 20.0 * log10(s2n) );
 	put_Status1(mfskmsg);
 }
 
@@ -506,7 +514,7 @@ void mfsk::eval_s2n(complex c, complex n)
 	s2n = decayavg( s2n, fabs((sig - noise) / noise), 8);
 }
 
-int mfsk::rx_process(double *buf, int len)
+int mfsk::rx_process(const double *buf, int len)
 {
 	complex z, *bins = 0;
 	int i;
@@ -540,6 +548,10 @@ int mfsk::rx_process(double *buf, int len)
 			if (counter++ == picturesize) {
 				counter = 0;
 				rxstate = RX_STATE_DATA;
+				// QUEUE_FLUSH();
+				put_status("");
+				string autosave_dir = HomeDir + "mfsk_pics/";
+				picRx->save_jpeg(autosave_dir.c_str());
 			} else
 				recvpic(z);
 			continue;
@@ -588,7 +600,7 @@ void mfsk::sendsymbol(int sym)
 {
 	double f, phaseincr;
 
-	f = tx_frequency - bandwidth / 2;
+	f = get_txfreq_woffset() - bandwidth / 2;
 	
 	sym = grayencode(sym & (numtones - 1));
 //printf("%5d", sym);
@@ -627,7 +639,7 @@ void mfsk::sendbit(int bit)
 
 void mfsk::sendchar(unsigned char c)
 {
-	char *code = varienc(c);
+	const char *code = varienc(c);
 	while (*code)
 		sendbit(*code++ - '0');
 	put_echo_char(c);
@@ -666,11 +678,11 @@ void mfsk::sendpic(unsigned char *data, int len)
 
 	for (i = 0; i < len; i++) {
 		if (txstate == TX_STATE_PICTURE)
-			updateTxPic(data[i]);
+			QUEUE(CMP_CB(&mfsk::updateTxPic, this, data[i])); //updateTxPic(data[i]);
 		if (reverse)
-			f = tx_frequency - bandwidth * (data[i] - 128) / 256.0;
+			f = get_txfreq_woffset() - bandwidth * (data[i] - 128) / 256.0;
 		else
-			f = tx_frequency + bandwidth * (data[i] - 128) / 256.0;
+			f = get_txfreq_woffset() + bandwidth * (data[i] - 128) / 256.0;
 			
 		for (j = 0; j < SAMPLES_PER_PIXEL; j++) {
 			*ptr++ = cos(phaseacc);
@@ -735,11 +747,11 @@ int mfsk::tx_process()
 				flushtx();
 				startpic = false;
 				txstate = TX_STATE_PICTURE_START;
-			} else if (xmtbyte == -1)
-				sendidle();
-			else if ( xmtbyte == 0x03 || stopflag) {
-				txstate = TX_STATE_FLUSH;
 			}
+			else if ( xmtbyte == 0x03 || stopflag)
+				txstate = TX_STATE_FLUSH;
+			else if (xmtbyte == -1)
+				sendidle();
 			else
 				sendchar(xmtbyte);
 			break;
@@ -752,6 +764,7 @@ int mfsk::tx_process()
 			rxstate = RX_STATE_DATA;
 			txstate = TX_STATE_PREAMBLE;
 			stopflag = false;
+			cwid();
 			return -1;
 
 		case TX_STATE_PICTURE_START:
@@ -771,12 +784,18 @@ int mfsk::tx_process()
 				else
 					sendpic( &xmtpicbuff[i], xmtbytes - i);
 				i += blocklen;
-				sprintf(mfskmsg,"Send picture: %.1f %%", (100.0 * i) / xmtbytes);
+				int n = snprintf(mfskmsg, sizeof(mfskmsg),
+						 "Send picture: %04.1f%% done",
+						 (100.0f * i) / xmtbytes);
+				print_time_left(xmtbytes - i, mfskmsg + n,
+						sizeof(mfskmsg) - n, ", ", " left");
 				put_status(mfskmsg);
 			}
+			QUEUE_FLUSH();
+
 			txstate = TX_STATE_DATA;
 			put_status("Send picture: done");
-			Fl::lock();
+			FL_LOCK_E();
 			btnpicTxSendAbort->hide();
 			btnpicTxSendColor->show();
 			btnpicTxSendGrey->show();
@@ -786,7 +805,7 @@ int mfsk::tx_process()
 			rxstate = RX_STATE_DATA;
 			counter = 0;
 			memset(picheader, ' ', sizeof(picheader));
-			Fl::unlock();
+			FL_UNLOCK_E();
 			break;
 	}
 
@@ -803,10 +822,11 @@ void mfsk::updateRxPic(unsigned char data, int pos)
 void cb_picRxClose( Fl_Widget *w, void *who)
 {
 	mfsk *me = (mfsk *)who;
-//	Fl::lock();
+//	FL_LOCK();
 	me->picRxWin->hide();
 	me->rxstate = mfsk::RX_STATE_DATA;
-//	Fl::unlock();
+	put_status("");
+//	FL_UNLOCK();
 }
 
 void cb_picRxSave( Fl_Widget *w, void *who)
@@ -826,9 +846,10 @@ void mfsk::makeRxViewer(int W, int H)
 	winH = H + 34;
 	picX = (winW - W) / 2;
 	picY = 2;
-	Fl::lock();
+	FL_LOCK_E();
 	if (!picRxWin) {
 		picRxWin = new Fl_Window(winW, winH);
+		picRxWin->xclass(PACKAGE_NAME);
 		picRx = new picture(picX, picY, W, H);
 		btnpicRxSave = new Fl_Button(winW/2 - 65, H + 6, 60, 24,"Save");
 		btnpicRxSave->callback(cb_picRxSave,this);
@@ -842,7 +863,7 @@ void mfsk::makeRxViewer(int W, int H)
 		picRx->clear();
 	}
 	picRxWin->show();
-	Fl::unlock();
+	FL_UNLOCK_E();
 }
 
 void mfsk::load_file(const char *n) {
@@ -884,11 +905,15 @@ void mfsk::load_file(const char *n) {
 	TxViewerResize(W, H);
 	
 // load the picture widget with the rgb image
-	Fl::lock();
+	FL_LOCK_D();
 	picTx->video(xmtimg, W * H * 3);
+	if (print_time_left(W * H * 3, txclr_tooltip, sizeof(txclr_tooltip), "Time needed: ") > 0)
+		btnpicTxSendColor->tooltip(txclr_tooltip);
 	btnpicTxSendColor->activate();
+	if (print_time_left(W * H, txgry_tooltip, sizeof(txgry_tooltip), "Time needed: ") > 0)
+		btnpicTxSendGrey->tooltip(txgry_tooltip);
 	btnpicTxSendGrey->activate();
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 void mfsk::updateTxPic(unsigned char data)
@@ -921,9 +946,9 @@ void cb_picTxLoad(Fl_Widget *,void *who) {
 void cb_picTxClose( Fl_Widget *w, void *who)
 {
 	mfsk *me = (mfsk *)who;
-	Fl::lock();
+	FL_LOCK_D();
 	me->picTxWin->hide();
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 void cb_picTxSendColor( Fl_Widget *w, void *who)
@@ -950,14 +975,14 @@ void cb_picTxSendColor( Fl_Widget *w, void *who)
 	my->col = 0;
 	my->row = 0;
 	my->pixelnbr = 0;
-	Fl::lock();
+	FL_LOCK_D();
 	my->btnpicTxSendColor->hide();
 	my->btnpicTxSendGrey->hide();
 	my->btnpicTxLoad->hide();
 	my->btnpicTxClose->hide();
 	my->btnpicTxSendAbort->show();
 	my->picTx->clear();
-	Fl::unlock();
+	FL_UNLOCK_D();
 // start the transmission
 	fl_lock(&trx_mutex);
 	if (trx_state != STATE_TX)
@@ -986,13 +1011,13 @@ void cb_picTxSendGrey( Fl_Widget *w, void *who)
 	my->row = 0;
 	my->pixelnbr = 0;
 	my->picTx->clear();
-	Fl::lock();
+	FL_LOCK_D();
 	my->btnpicTxSendColor->hide();
 	my->btnpicTxSendGrey->hide();
 	my->btnpicTxLoad->hide();
 	my->btnpicTxClose->hide();
 	my->btnpicTxSendAbort->show();
-	Fl::unlock();
+	FL_UNLOCK_D();
 // start the transmission
 	fl_lock(&trx_mutex);
 	if (trx_state != STATE_TX)
@@ -1008,9 +1033,9 @@ void cb_picTxSendAbort( Fl_Widget *w, void *who)
 	mfsk *my = (mfsk *)who;
 	my->abortxmt = true;
 // reload the picture widget with the rgb image
-	Fl::lock();
+	FL_LOCK_D();
 	my->picTx->video(my->xmtimg, my->TxImg->w() * my->TxImg->h() * 3);
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 void mfsk::TxViewerResize(int W, int H)
@@ -1021,7 +1046,7 @@ void mfsk::TxViewerResize(int W, int H)
 	winH = H < 180 ? 180 : H + 30;
 	picX = (winW - W) / 2;
 	picY =  (winH - 30 - H)/2;
-	Fl::lock();
+	FL_LOCK_D();
 	picTxWin->size(winW, winH);
 	picTx->resize(picX, picY, W, H);
 	picTx->clear();
@@ -1030,7 +1055,7 @@ void mfsk::TxViewerResize(int W, int H)
 	btnpicTxSendAbort->resize(winW/2 - 123, winH - 28, 122, 24);
 	btnpicTxLoad->resize(winW/2 + 1, winH - 28, 60, 24);
 	btnpicTxClose->resize(winW/2 + 63, winH - 28, 60, 24);
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 void mfsk::makeTxViewer(int W, int H)
@@ -1041,9 +1066,10 @@ void mfsk::makeTxViewer(int W, int H)
 	winH = H < 180 ? 180 : H + 30;
 	picX = (winW - W) / 2;
 	picY =  2;
-	Fl::lock();
+	FL_LOCK_D();
 	if (!picTxWin) {
 		picTxWin = new Fl_Window(winW, winH);
+		picTxWin->xclass(PACKAGE_NAME);
 		picTx = new picture (picX, picY, W, H);
 		btnpicTxSendColor = new Fl_Button(winW/2 - 123, winH - 28, 60, 24, "XmtClr");
 		btnpicTxSendColor->callback(cb_picTxSendColor, this);
@@ -1073,6 +1099,19 @@ void mfsk::makeTxViewer(int W, int H)
 		btnpicTxSendAbort->hide();
 	}
 	picTxWin->show();
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
+int mfsk::print_time_left(size_t bytes, char *str, size_t len,
+			  const char *prefix, const char *suffix)
+{
+	float time_sec = bytes * 0.001;
+	int time_min = (int)(time_sec / 60);
+	time_sec -= time_min * 60;
+
+	if (time_min)
+		return snprintf(str, len, "%s%02dm%2.1fs%s",
+				prefix, time_min, time_sec, suffix);
+	else
+		return snprintf(str, len, "%s%2.1fs%s", prefix, time_sec, suffix);
+}
