@@ -3,9 +3,11 @@
  */
 
 
+#include <config.h>
+
 #include "log.h"
 
-static 	char *lognames[] = {"RX", "TX"};
+static const char *lognames[] = {"RX", "TX"};
 
 cLogfile::cLogfile(string fname) {
 	logtype = LOG_RX;
