@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include "macros.h"
 #include "macroedit.h"
 
@@ -35,6 +37,8 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add("<LDT>\tLocal datetime");
 	w->add("<ZDT>\tZulu datetime");
 	w->add("<ID>\tMode ID'r");
+	w->add("<TEXT>\tVideo text");
+	w->add("<CWID>\tCW identifier");
 	w->add("<RX>\treceive");
 	w->add("<TX>\ttransmit");
 	w->add("<VER>\tFldigi + version");
@@ -49,11 +53,11 @@ void cbMacroEditOK(Fl_Widget *w, void *)
 {
 	macros.text[iMacro] = macrotext->value();
 	macros.name[iMacro] = labeltext->value();
-	Fl::lock();
+	FL_LOCK_D();
 		int n = iMacro;
 		if (n >= 10) n -= 10;
 		btnMacro[n]->label( macros.name[iMacro].c_str());
-	Fl::unlock();
+	FL_UNLOCK_D();
 	MacroEditDialog->hide();
 	macros.changed = true;
 }
@@ -108,6 +112,7 @@ void editMacro(int n)
 	macrotext->value(macros.text[n].c_str());
 	labeltext->value(macros.name[n].c_str());
 	iMacro = n;
+	MacroEditDialog->xclass(PACKAGE_NAME);
 	MacroEditDialog->show();
 }
 
