@@ -1,3 +1,5 @@
+#include <config.h>
+
 #include "macros.h"
 
 #include "main.h"
@@ -5,7 +7,6 @@
 #include "fl_digi.h"
 #include "configuration.h"
 #include "Config.h"
-#include "version.h"
 #include "logger.h"
 
 #include <string>
@@ -114,16 +115,16 @@ CQ CQ CQ de <MYCALL> <MYCALL> <MYCALL> pse k\n\
 }
 
 struct paldata {
-	char *fname;
-	char *rgbstr0;
-	char *rgbstr1;
-	char *rgbstr2;
-	char *rgbstr3;
-	char *rgbstr4;
-	char *rgbstr5;
-	char *rgbstr6;
-	char *rgbstr7;
-	char *rgbstr8;
+	const char *fname;
+	const char *rgbstr0;
+	const char *rgbstr1;
+	const char *rgbstr2;
+	const char *rgbstr3;
+	const char *rgbstr4;
+	const char *rgbstr5;
+	const char *rgbstr6;
+	const char *rgbstr7;
+	const char *rgbstr8;
 };
 
 paldata palfiles[] = {
@@ -258,7 +259,6 @@ void newpalettes()
 void createDotFldigi()
 {
 	string Filename = HomeDir;
-	mkdir(Filename.c_str(), O_CREAT | S_IRUSR|S_IWUSR|S_IXUSR);
 	Filename.append("macros.mdf");
 	newmacros();
 	macros.saveMacros(Filename);
