@@ -25,6 +25,8 @@
 //
 //
 
+#include <config.h>
+
 #include <string>
 #include <FL/Fl.H>
 #include <FL/fl_ask.H>
@@ -38,7 +40,7 @@ using namespace std;
 
 double olivia::nco(double freq)
 {
-    preamblephase += 2.0 * M_PI * freq / 8000;
+    preamblephase += 2.0 * M_PI * freq / samplerate;
 
 	if (preamblephase > M_PI)
 		preamblephase -= 2.0 * M_PI;
@@ -58,7 +60,7 @@ void olivia::tx_init(cSound *sc)
 
 	preamblesent = 0;
 	postamblesent = 0;
-	txbasefreq = tx_frequency;
+	txbasefreq = get_txfreq_woffset();
 
 	Rx->Flush();
 
@@ -73,12 +75,7 @@ void olivia::tx_init(cSound *sc)
 		Tx->Reverse = 0; 
 	}
 
-	if (trx_state != STATE_TUNE && progdefaults.sendid == true)
-		wfid->transmit(mode);
-	else if (trx_state != STATE_TUNE && progdefaults.macroid == true) {
-		wfid->transmit(mode);
-		progdefaults.macroid = false;
-	}
+	videoText();
 
 	Tx->Preset();
 	Tx->Start();
@@ -88,7 +85,7 @@ void olivia::tx_init(cSound *sc)
 void olivia::send_preamble()
 {
 	double freqa, freqb;
-    int i;
+    int i, sr4 = samplerate / 4;
 
 	if (reverse) { 
 		freqa = txbasefreq + (bandwidth / 2.0); 
@@ -98,25 +95,21 @@ void olivia::send_preamble()
 		freqb = txbasefreq + (bandwidth / 2.0); 
 	}
 
-	for (i = 0; i < 2000; i++)
+	for (i = 0; i < sr4; i++)
 		outbuf[i] = nco(freqa);
-
-	for (i = 2000; i < 4000; i++)
+	for (i = sr4; i < 2*sr4; i++)
 		outbuf[i] = nco(freqb);
-
-	for (i = 4000; i < 6000; i++)
+	for (i = 2*sr4; i < 3*sr4; i++)
 		outbuf[i] = nco(freqa);
-
-	for (i = 6000; i < 8000; i++)
+	for (i = 3*sr4; i < samplerate; i++)
 		outbuf[i] = nco(freqb);
-
-	ModulateXmtr(outbuf, 8000);
+	ModulateXmtr(outbuf, samplerate);
 }
 
 void olivia::send_postamble()
 {
 	double freqa, freqb;
-	int i;
+	int i, sr4 = samplerate / 4;
 
 	if (reverse) { 
 		freqa = txbasefreq + (bandwidth / 2.0); 
@@ -126,19 +119,15 @@ void olivia::send_postamble()
 		freqb = txbasefreq + (bandwidth / 2.0); 
 	}
 
-	for (i = 0; i < 1000; i++)
+	for (i = 0; i < sr4; i++)
 		outbuf[i] = nco(freqa);
-
-	for (i = 1000; i < 2000; i++)
+	for (i = sr4; i < 2*sr4; i++)
 		outbuf[i] = nco(freqb);
-
-	for (i = 2000; i < 3000; i++)
+	for (i = 2*sr4; i < 3*sr4; i++)
 		outbuf[i] = nco(freqa);
-
-	for (i = 3000; i < 4000; i++)
+	for (i = 3*sr4; i < samplerate; i++)
 		outbuf[i] = nco(freqb);
-
-	ModulateXmtr(outbuf, 4000);
+	ModulateXmtr(outbuf, samplerate);
 }
 
 void olivia::rx_init()
@@ -186,6 +175,8 @@ int olivia::tx_process()
 			Tx->Stop();
 		} else {
 			/* Replace un-representable characters with a dot */
+			if (c == -1)
+                                c = 0;
 			if (c > (olivia_esc ? 255 : 127))
 				c = '.';
 			if (c > 127) {
@@ -194,7 +185,7 @@ int olivia::tx_process()
 			}
 			Tx->PutChar(c);
 		}
-    }
+	}
 
 	if (Tx->GetChar(ch) > 0)
 		if ((c = unescape(ch)) != -1)
@@ -213,20 +204,22 @@ int olivia::tx_process()
 		send_postamble();
 	}
 
-	if (!Tx->Running())
+	if (!Tx->Running()) {
+		cwid();
 		return -1;
+	}
 
 	return 0;
 }
 
-int olivia::rx_process(double *buf, int len)
+int olivia::rx_process(const double *buf, int len)
 {
 	int i, c;
 	unsigned char ch = 0;
 	double snr;
 	static char msg1[20];
 	static char msg2[20];
-	static char msg3[60];
+//	static char msg3[60];
 
 	if ((lastfreq != frequency || Rx->Reverse != 0) && !reverse) {
 		Rx->FirstCarrierMultiplier = (frequency - (Rx->Bandwidth / 2)) / 500; 
@@ -265,9 +258,9 @@ int olivia::rx_process(double *buf, int len)
 	put_Status1(msg1);
 	sprintf(msg2, "Freq: %+4.1f", Rx->FrequencyOffset());
 	put_Status2(msg2);
-	sprintf(msg3, "Tune Margin: %4.1f, Time: %5.3f Hz, Block Period: %5.3f sec",
-		Rx->TuneMargin(), Rx->TimeOffset(), Rx->BlockPeriod());
-	put_status(msg3);
+//	sprintf(msg3, "Tune: %4.1f, Time: %5.3f Hz, Block: %5.3f sec",
+//		Rx->TuneMargin(), Rx->TimeOffset(), Rx->BlockPeriod());
+//	put_status(msg3);
 
 	while (Rx->GetChar(ch) > 0)
 		if ((c = unescape(ch)) != -1 && c > 7)
@@ -280,13 +273,14 @@ void olivia::restart()
 {
 	tones	= progdefaults.oliviatones;
 	bw 		= progdefaults.oliviabw;
-	samplerate = 8000;
+//	samplerate = 8000;
+//	samplerate = 11025;
 	
 	Tx->Tones = 2 * (1 << tones);
 	Tx->Bandwidth = 125 * (1 << bw);
-	Tx->SampleRate = samplerate;
-	Tx->OutputSampleRate = samplerate; //samplerate;
-    txbasefreq = tx_frequency;
+	Tx->SampleRate = 8000.0;//samplerate;
+	Tx->OutputSampleRate = samplerate;
+    txbasefreq = get_txfreq_woffset();
 
 	if (reverse) { 
 		Tx->FirstCarrierMultiplier = (txbasefreq + (Tx->Bandwidth / 2)) / 500; 
@@ -300,7 +294,6 @@ void olivia::restart()
 		fl_message("olivia: transmitter preset failed!");
 		return;
 	}
-//
 		
 	txbufferlen = Tx->MaxOutputLen;
 	if (txbuffer) delete [] txbuffer;
@@ -311,7 +304,6 @@ void olivia::restart()
 
 	rxbufferlen = 0; //SCBLOCKSIZE;
 	rxbuffer = 0; //new short int[rxbufferlen];
-//
 	
 	Rx->Tones = Tx->Tones;
 	Rx->Bandwidth = Tx->Bandwidth;
@@ -319,8 +311,8 @@ void olivia::restart()
 	Rx->SyncIntegLen = sinteg;
 	Rx->SyncThreshold = squelchon ? squelch : 0.0;
 
-	Rx->SampleRate = samplerate;
-	Rx->InputSampleRate = samplerate; //samplerate;
+	Rx->SampleRate = 8000.0;//samplerate;
+	Rx->InputSampleRate = samplerate;
 
 	if (reverse) { 
 		Rx->FirstCarrierMultiplier = (frequency + (Rx->Bandwidth / 2)) / 500; 
@@ -336,6 +328,8 @@ void olivia::restart()
 	}
 	fragmentsize = 1024;
 	set_bandwidth(Tx->Bandwidth);
+	
+//	Rx->PrintParameters();
 
 	put_MODEstatus(mode);
 }
@@ -349,18 +343,19 @@ void olivia::init()
 
 olivia::olivia()
 {
-	Fl::lock();
-		Tx = new MFSK_Transmitter< float >;
-		Rx = new MFSK_Receiver< float >;
-	Fl::unlock();
-
 	txbuffer = 0;
 	txfbuffer = 0;
 	rxbuffer = 0;
+//	samplerate = 11025;
 	samplerate = 8000;
 
+	FL_LOCK();
+		Tx = new MFSK_Transmitter< float >;
+		Rx = new MFSK_Receiver< float >;
+	FL_UNLOCK();
+
 	mode = MODE_OLIVIA;
-	wfid = new id(this);
+//	wfid = new id(this);
 	init();
 }
 
@@ -371,6 +366,6 @@ olivia::~olivia()
 	if (txbuffer) delete [] txbuffer;
 	if (txfbuffer) delete [] txfbuffer;
 	if (rxbuffer) delete [] rxbuffer;
-	if (wfid) delete wfid;
+//	if (wfid) delete wfid;
 }
 
