@@ -1,5 +1,14 @@
 //
 // hamlib.cxx  --  Hamlib (rig control) interface for fldigi
+#include <config.h>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <termios.h>
+#include <string>
+
 #include "configuration.h"
 #include "Config.h"
 #include <FL/fl_ask.H>
@@ -35,16 +44,54 @@ static long int hamlib_freq;
 static rmode_t hamlib_rmode = RIG_MODE_USB;
 static pbwidth_t hamlib_pbwidth = 3000;
 
+static int ioctl_bits;
+
 static int dummy = 0;
 
 static void *hamlib_loop(void *args);
 
-void show_error(char * a, char * b)
+void show_error(const char * a, const char * b)
 {
 	string msg = a;
 	msg.append(": ");
 	msg.append(b);
 	put_status((char*)msg.c_str());
+	std::cout << msg.c_str() << std::endl; std::cout.flush();
+}
+
+bool hamlib_setRTSDTR()
+{
+	if (progdefaults.RTSplus == false && progdefaults.DTRplus == false)
+		return true;
+		
+	int hamlibfd =  open(progdefaults.HamRigDevice.c_str(), O_RDWR | O_NOCTTY | O_NDELAY);
+
+	if (hamlibfd < 0) {
+		show_error("Cannot open PTT device ", progdefaults.HamRigDevice.c_str());
+		return false;
+	}
+
+	ioctl(hamlibfd, TIOCMGET, &ioctl_bits);
+
+	if (progdefaults.RTSplus)
+		ioctl_bits |= TIOCM_RTS;		// set RTS bit
+	else
+		ioctl_bits &= ~TIOCM_RTS;		// clear RTS bit
+	if (progdefaults.DTRplus)
+		ioctl_bits |= TIOCM_DTR;		// set DTR bit
+	else
+		ioctl_bits &= ~TIOCM_DTR;		// clear DTR bit
+std::cout << "TIOCM_RTS " << TIOCM_RTS << ", RTSplus " << progdefaults.RTSplus;
+std::cout << std::endl;
+std::cout << "TIOCM_DTR " << TIOCM_DTR << ", DTRplus " << progdefaults.DTRplus;
+std::cout << std::endl;
+std::cout << ioctl_bits;
+	ioctl(hamlibfd, TIOCMSET, &ioctl_bits);
+	close(hamlibfd);
+	
+std::cout << "=> setRTSDTR() => " << ioctl_bits << std::endl; std::cout.flush();
+	
+	return true;
 }
 
 bool hamlib_init(bool bPtt)
@@ -57,8 +104,19 @@ bool hamlib_init(bool bPtt)
 	string	port, spd;
 	
 	hamlib_ptt = bPtt;
+	hamlib_closed = true;
+	
+	if (progdefaults.HamRigDevice == "COM1")
+		port = "/dev/ttyS0";
+	else if (progdefaults.HamRigDevice == "COM2")
+		port = "/dev/ttyS1";
+	else if (progdefaults.HamRigDevice == "COM3")
+		port = "/dev/ttyS2";
+	else if (progdefaults.HamRigDevice == "COM4")
+		port = "/dev/ttyS3";
+	else
+		port = progdefaults.HamRigDevice;
 
-	port = progdefaults.HamRigDevice;
 	spd = progdefaults.strBaudRate();
 
 	list<string>::iterator pstr = (xcvr->rignames).begin();
@@ -77,11 +135,19 @@ bool hamlib_init(bool bPtt)
 		return -1;
 	}
 
+//	if (hamlib_setRTSDTR() == false) {
+//		return -1;
+//	}
+	
 	try {
 		model = (*prig)->rig_model;
 		xcvr->init(model);
 		xcvr->setConf("rig_pathname", port.c_str());
 		xcvr->setConf("serial_speed", spd.c_str());
+		if (progdefaults.RTSplus)
+			xcvr->setConf("rts_state", "ON");
+		if (progdefaults.DTRplus)
+			xcvr->setConf("dtr_state", "ON");
 		xcvr->open();
 	}
 	catch (RigException Ex) {
@@ -89,11 +155,22 @@ bool hamlib_init(bool bPtt)
 		return -1;
 	}
 
-	MilliSleep(100);
+//	if (hamlib_setRTSDTR() == false)
+//		return -1;
+		
+	MilliSleep(200);
+//char temp[80];
+//xcvr->getConf("dtr_state", temp);
+//std::cout << "Hamlib DTR " << temp << std::endl; std::cout.flush();
 
 	try {
 		need_freq = true;
 		freq = xcvr->getFreq();
+		if (freq == 0) {
+			xcvr->close();
+			show_error("Transceiver not responding", "");
+			return -1;
+		}
 	}
 	catch (RigException Ex) {
 		show_error("Get Freq",Ex.message);
@@ -122,14 +199,13 @@ bool hamlib_init(bool bPtt)
 	}
 
 	hamlib_freq = 0;
-	hamlib_rmode = RIG_MODE_USB;
+	hamlib_rmode = RIG_MODE_NONE;//RIG_MODE_USB;
 
 	if (fl_create_thread(hamlib_thread, hamlib_loop, &dummy) < 0) {
-		show_error("Hamlib init:", "pthread_create failed");
+		std::cout << "Hamlib init:  pthread_create failed\n"; std::cout.flush();
 		xcvr->close();
 		return false;
 	} 
-//	std::cout << "Hamlib on" << std::endl; fflush(stdout);
 
 	init_Hamlib_RigDialog();
 	
@@ -140,25 +216,25 @@ bool hamlib_init(bool bPtt)
 
 void hamlib_close(void)
 {
-	int count = 100;
-	if (xcvr->isOnLine() == false || hamlib_closed == true)
+	int count = 20;
+	if (xcvr->isOnLine() == false)
 		return;
 
+	if (hamlib_closed == true)
+		return;
+		
 	hamlib_exit = true;
 
-//	std::cout << "Waiting for hamlib to exit "; fflush(stdout);
 	while (hamlib_closed == false) {
-//		std::cout << "."; fflush(stdout);
+//		std::cout << "."; cout.flush();
 		MilliSleep(50);
 		count--;
 		if (!count) {
-			std::cout << "\nHamlib stuck\n"; fflush(stdout);
+			std::cout << "\nHamlib stuck\n"; cout.flush();
+			xcvr->close();
 			exit(0);
 		}
 	}
-
-	xcvr->close();
-//	std::cout << "\nexit OK" << std::endl; fflush(stdout);
 }
 
 bool hamlib_active(void)
@@ -184,42 +260,28 @@ void hamlib_set_ptt(int ptt)
 	fl_unlock(&hamlib_mutex);
 }
 
-void hamlib_set_qsy(void)
+void hamlib_set_qsy(long long f, long long fmid)
 {
-	long int fmid;
-	long int f;
 	if (xcvr->isOnLine() == false) 
 		return;
 	fl_lock(&hamlib_mutex);
-// this is the sweet spot frequency that is the center of the PBF in the rig
-		if (active_modem->get_mode() == MODE_CW)
-			fmid = (long int)progdefaults.CWsweetspot;
-		else if (active_modem->get_mode() == MODE_RTTY)
-			fmid = (long int)progdefaults.RTTYsweetspot;
-		else
-			fmid = (long int)progdefaults.PSKsweetspot;
-		f = wf->rfcarrier();
-		if (wf->USB())
-			f += (wf->carrier() - fmid);
-		else
-			f -= (wf->carrier() - fmid);
-		double fdbl = f;
-		hamlib_qsy = false;
-		try {
-			xcvr->setFreq(fdbl);
-			if (active_modem->freqlocked() == true) {
-				active_modem->set_freqlock(false);
-				active_modem->set_freq((int)fmid);
-				active_modem->set_freqlock(true);
-			} else
-				active_modem->set_freq((int)fmid);
-			wf->rfcarrier(f);
-			wf->movetocenter();
-		}
-		catch (RigException Ex) {
-			show_error("QSY", Ex.message);
-			hamlib_passes = 0;
-		}
+	double fdbl = f;
+	hamlib_qsy = false;
+	try {
+		xcvr->setFreq(fdbl);
+		if (active_modem->freqlocked() == true) {
+			active_modem->set_freqlock(false);
+			active_modem->set_freq((int)fmid);
+			active_modem->set_freqlock(true);
+		} else
+			active_modem->set_freq((int)fmid);
+		wf->rfcarrier(f);
+		wf->movetocenter();
+	}
+	catch (RigException Ex) {
+		show_error("QSY", Ex.message);
+		hamlib_passes = 0;
+	}
 	fl_unlock(&hamlib_mutex);
 }
 
@@ -227,11 +289,10 @@ int hamlib_setfreq(long f)
 {
 	if (xcvr->isOnLine() == false)
 		return -1;
-	hamlib_freq = f;
 	fl_lock(&hamlib_mutex);
 		try {
-			xcvr->setFreq(hamlib_freq);
-			wf->rfcarrier(hamlib_freq);
+			xcvr->setFreq(f);
+			wf->rfcarrier(f);//(hamlib_freq);
 		}
 		catch (RigException Ex) {
 		show_error("SetFreq", Ex.message);
@@ -289,11 +350,14 @@ pbwidth_t hamlib_getwidth()
 
 static void *hamlib_loop(void *args)
 {
+	SET_THREAD_ID(RIGCTL_TID);
+
 	long int freq = 0L;
 	rmode_t  numode = RIG_MODE_USB;
 	bool freqok = false, modeok = false;
 	
 	for (;;) {
+loop:
 		MilliSleep(100);
 		if (hamlib_exit)
 			break;
@@ -308,20 +372,30 @@ static void *hamlib_loop(void *args)
 				f = xcvr->getFreq();
 				freq = (long int) f;
 				freqok = true;
+				if (freq == 0) {
+					fl_unlock (&hamlib_mutex);
+					goto loop;
+//					hamlib_exit = true;
+				}
 			}
 			catch (RigException Ex) {
-				show_error("Hamlib loop Get Freq", Ex.message);
+				show_error("No transceiver comms", "");
 				freqok = false;
+				hamlib_exit = true;
 			}
 		}
+		if (hamlib_exit)
+			break;
+			
 		if (need_mode) {
 			try {
 				numode = xcvr->getMode(hamlib_pbwidth);
 				modeok = true;
 			}
 			catch (RigException Ex) {
-				show_error("Hamlib loop Get Mode", Ex.message);
+				show_error("No transceiver comms", "");
 				modeok = false;
+				hamlib_exit = true;
 			}
 		}
 		fl_unlock (&hamlib_mutex);
@@ -333,9 +407,9 @@ static void *hamlib_loop(void *args)
 
 		if (freqok && freq && (freq != hamlib_freq)) {
 			hamlib_freq = freq;
-			Fl::lock();
+			FL_LOCK();
 				FreqDisp->value(hamlib_freq);
-			Fl::unlock();
+			FL_UNLOCK();
 			wf->rfcarrier(hamlib_freq);
 		}
 		
@@ -360,7 +434,21 @@ static void *hamlib_loop(void *args)
 		if (hamlib_exit)
 			break;
 	}
+
+	xcvr->close();
 	hamlib_closed = true;
+
+	if (rigcontrol)
+		rigcontrol->hide();
+	wf->USB(true);
+	wf->rfcarrier(atoi(cboBand->value())*1000L);
+	wf->setQSY(0);
+	FL_LOCK();
+	cboBand->show();
+	btnSideband->show();
+	activate_rig_menu_item(false);
+	FL_UNLOCK();
+
 	return NULL;
 }
 
