@@ -5,17 +5,23 @@
 //
 // ====================================================================
 
+#include <config.h>
+
 #include <string>
 
-#include "fl_digi.h"
-#include "misc.h"
+#ifdef RIGCATTEST
+	#include "rigCAT.h"
+#else
+	#include "fl_digi.h"
+	#include "misc.h"
+	#include "configuration.h"
+#endif
 
 #include "rigsupport.h"
 #include "rigdialog.h"
 #include "rigxml.h"
 #include "serial.h"
 #include "rigio.h"
-#include "configuration.h"
 
 #include "threads.h"
 
@@ -749,7 +755,8 @@ void rigCAT_sendINIT()
 	fl_unlock(&rigCAT_mutex);
 }
 
-static void show_error(char * a, char * b)
+__attribute__((unused))
+static void show_error(const char * a, const char * b)
 {
 	string msg = a;
 	msg.append(": ");
@@ -765,24 +772,29 @@ bool rigCAT_init()
 	}
 
 	if (readRigXML() == false) {
-		show_error("No rig.xml file present","");
+		std::cout << "No rig.xml file present\n"; std::cout.flush();
 		return false;
 	}
 
 	if (rigio.OpenPort() == false) {
-		show_error("Cannot open serial port", (char *)rigio.Device().c_str());
+		std::cout << "Cannot open serial port " << (char *)rigio.Device().c_str();
+		std::cout << std::endl; std::cout.flush();
 		return false;
 	}
 	llFreq = 0;
 	sRigMode = "";
 	sRigWidth = "";
 	
+	if (rigCAT_getfreq() <= 0) {
+		std::cout << "Transceiver not responding\n"; std::cout.flush();
+		return false;
+	}
+	
 	if (fl_create_thread(rigCAT_thread, rigCAT_loop, &dummy) < 0) {
-		show_error("rig init:", "pthread_create failed");
+		std::cout << "rig init: pthread_create failed\n"; std::cout.flush();
 		rigio.ClosePort();
 		return false;
 	} 
-//	std::cout << "RigCAT on" << std::endl; fflush(stdout);
 
 	init_Xml_RigDialog();
 	rigCAT_sendINIT();
@@ -805,13 +817,13 @@ void rigCAT_close(void)
 		count--;
 		if (!count) {
 			std::cout << "\nRigCAT stuck\n"; fflush(stdout);
+		fl_lock(&rigCAT_mutex);
+			rigio.ClosePort();
+		fl_unlock(&rigCAT_mutex);
 			exit(0);
 		}
 	}
 	
-	fl_lock(&rigCAT_mutex);
-		rigio.ClosePort();
-	fl_unlock(&rigCAT_mutex);
 //	std::cout << "\nExit OK" << std::endl; fflush(stdout);
 }
 
@@ -833,37 +845,25 @@ void rigCAT_set_ptt(int ptt)
 	fl_unlock(&rigCAT_mutex);
 }
 
-void rigCAT_set_qsy(void)
+#ifndef RIGCATTEST
+void rigCAT_set_qsy(long long f, long long fmid)
 {
-	long long fmid;
-	long long f;
 	if (rigCAT_open == false)
 		return;
-// this is the sweet spot frequency that is the center of the PBF in the rig
-		if (active_modem->get_mode() == MODE_CW)
-			fmid = (long long)progdefaults.CWsweetspot;
-		else if (active_modem->get_mode() == MODE_RTTY)
-			fmid = (long long)progdefaults.RTTYsweetspot;
-		else
-			fmid = (long long)progdefaults.PSKsweetspot;
-		f = wf->rfcarrier();
-		if (wf->USB())
-			f += (wf->carrier() - fmid);
-		else
-			f -= (wf->carrier() - fmid);
-// send new freq to rig
-		rigCAT_setfreq(f);
-		
-		if (active_modem->freqlocked() == true) {
-			active_modem->set_freqlock(false);
-			active_modem->set_freq((int)fmid);
-			active_modem->set_freqlock(true);
-		} else
-			active_modem->set_freq((int)fmid);
-		wf->rfcarrier(f);
-		wf->movetocenter();
-}
 
+	// send new freq to rig
+	rigCAT_setfreq(f);
+
+	if (active_modem->freqlocked() == true) {
+		active_modem->set_freqlock(false);
+		active_modem->set_freq((int)fmid);
+		active_modem->set_freqlock(true);
+	} else
+		active_modem->set_freq((int)fmid);
+	wf->rfcarrier(f);
+	wf->movetocenter();
+}
+#endif
 
 bool ModeIsLSB(string s)
 {
@@ -878,6 +878,8 @@ bool ModeIsLSB(string s)
 
 static void *rigCAT_loop(void *args)
 {
+	SET_THREAD_ID(RIGCTL_TID);
+
 	long long freq = 0L;
 	string sWidth, sMode;
 
@@ -893,6 +895,9 @@ static void *rigCAT_loop(void *args)
 		fl_lock(&rigCAT_mutex);
 			freq = rigCAT_getfreq();
 		fl_unlock(&rigCAT_mutex);
+		if (freq <= 0)
+			goto loop3;
+			
 		MilliSleep(10);
 		if (rigCAT_exit == true)
 			goto exitloop;
@@ -919,28 +924,32 @@ static void *rigCAT_loop(void *args)
 		
 		if (freq && freq != llFreq) {
 			llFreq = freq;
-			Fl::lock();
+			FL_LOCK();
 				FreqDisp->value(freq);
-			Fl::unlock();
+			FL_UNLOCK();
+#ifndef RIGCATTEST
 			wf->rfcarrier(freq);
+#endif
 		}
 		if (sWidth.size() && sWidth != sRigWidth) {
 			sRigWidth = sWidth;
-			Fl::lock();
+			FL_LOCK();
 				opBW->value(sWidth.c_str());
-			Fl::unlock();
+			FL_UNLOCK();
 		}
 		if (sMode.size() && sMode != sRigMode) {
 			sRigMode = sMode;
+#ifndef RIGCATTEST
 			if (ModeIsLSB(sMode))
 				wf->USB(false);
 			else
 				wf->USB(true);
-			Fl::lock();
+#endif
+			FL_LOCK();
 				opMODE->value(sMode.c_str());
-			Fl::unlock();
+			FL_UNLOCK();
 		}
-//		Fl::awake();
+//		FL_AWAKE();
 		goto loop0;
 loop3:
 		MilliSleep(10);
@@ -954,6 +963,21 @@ loop0:
 exitloop:
 	rigCAT_open = false;
 	rigCAT_exit = false;
+	if (rigcontrol)
+		rigcontrol->hide();
+	wf->USB(true);
+	wf->rfcarrier(atoi(cboBand->value())*1000L);
+	wf->setQSY(0);
+	FL_LOCK();
+	cboBand->show();
+	btnSideband->show();
+	activate_rig_menu_item(false);
+	FL_UNLOCK();
+
+	fl_lock(&rigCAT_mutex);
+		rigio.ClosePort();
+	fl_unlock(&rigCAT_mutex);
+
 	return NULL;
 }
 
