@@ -8,6 +8,8 @@
 //
 //########################################################################
 
+#include <config.h>
+
 #include "rigdialog.h"
 #include "rigsupport.h"
 #include "rigxml.h"
@@ -26,8 +28,8 @@ string windowTitle;
 long int freqlist[LISTSIZE];
 int  numinlist = 0;
 
-#ifndef NOHAMLIB
-char *szmodes[] = {
+#if USE_HAMLIB
+const char *szmodes[] = {
 	"AM",
 	"USB",
 	"LSB",
@@ -187,6 +189,7 @@ void buildlist()
 {
 	if (readFreqList() == true)
 		return;
+	Fl::lock();
 	for (int n = 0; n < 100; n++) {freqlist[n] = 0;}
 	addtoList (1807000L);
 	addtoList (10135000L);
@@ -206,10 +209,11 @@ void buildlist()
 	addtoList (7030000L);
 	updateSelect();
 	FreqDisp->value(freqlist[0]);
+	Fl::unlock();
 }
 
 void setMode() {
-#ifndef NOHAMLIB
+#if USE_HAMLIB
 	if (progdefaults.chkUSEHAMLIBis)
 		hamlib_setmode(modes[opMODE->index()-1]);
 	else
@@ -224,7 +228,7 @@ void setBW() {
 int movFreq() {
 	long int f;
 	f = FreqDisp->value();
-#ifndef NOHAMLIB
+#if USE_HAMLIB
 	if (progdefaults.chkUSEHAMLIBis)
 		hamlib_setfreq(f);
 	else
@@ -272,12 +276,12 @@ bool init_Xml_RigDialog()
 	return true;
 }
 
-#ifndef NOHAMLIB
+#if USE_HAMLIB
 bool init_Hamlib_RigDialog()
 {
 	opBW->hide();
 	opMODE->clear();
-	char **p;
+	const char **p;
 	p = szmodes;
 	while (*p) 
 		opMODE->add(*p++);
@@ -292,6 +296,7 @@ Fl_Double_Window *createRigDialog()
 {
 	Fl_Double_Window *w;
 	w = rig_dialog();
+	w->xclass(PACKAGE_NAME);
 	return w;
 }
 
