@@ -8,10 +8,19 @@
 //
 //########################################################################
 
+#include <config.h>
+
+#include <iostream>
+#include <fstream>
+
 #include "rigio.h"
 #include "rigxml.h"
 #include "rigsupport.h"
-#include "main.h"
+#ifdef RIGCATTEST
+	#include "rigCAT.h"
+#else
+	#include "main.h"
+#endif
 
 //#define DEBUGXML
 
@@ -823,7 +832,7 @@ void parseXML()
 
 bool readRigXML()
 {
-	char szLine[256];
+	char szLine[256] = "";
 	int lines = 0;
 
 	commands.clear();
