@@ -25,6 +25,8 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <iostream>
 
@@ -42,12 +44,7 @@ void  throb::tx_init(cSound *sc)
 	scard = sc;
 	preamble = 4;
 	reset_syms();
-	if (trx_state != STATE_TUNE && progdefaults.sendid == true)
-		wfid->transmit(mode);
-	else if (trx_state != STATE_TUNE && progdefaults.macroid == true) {
-		wfid->transmit(mode);
-		progdefaults.macroid = false;
-	}
+	videoText();
 }
 
 void  throb::rx_init()
@@ -79,7 +76,7 @@ throb::~throb()
 	if (outbuf) delete[] outbuf;
 	for (int i = 0; i < num_tones; i++)
 		if (rxtone[i]) delete [] rxtone[i];
-	if (wfid) delete wfid;
+//	if (wfid) delete wfid;
 }
 
 void throb::flip_syms() //call this whenever a space or idle is sent or received
@@ -121,7 +118,7 @@ void throb::reset_syms() //call when switching from TX to RX or vice versa
 throb::throb(trx_mode throb_mode) : modem()
 {
 	double bw;
-	double *fp;
+	double *fp = 0;
 
 	mode = throb_mode;
 
@@ -155,6 +152,7 @@ throb::throb(trx_mode throb_mode) : modem()
 	case MODE_THROB4:
 	default:
 		symlen = SYMLEN_4;
+		mk_full_pulse(symlen);
 		txpulse = mk_full_pulse(symlen);
 		fp = mk_full_pulse(symlen / DOWN_SAMPLE);
 		num_tones = 9;
@@ -172,8 +170,8 @@ throb::throb(trx_mode throb_mode) : modem()
                 fp = mk_semi_pulse(symlen / DOWN_SAMPLE);
                 num_tones = 11;
                 num_chars = 55;
-		idlesym = 0;
-		spacesym = 1;
+				idlesym = 0;
+				spacesym = 1;
                 for (int i = 0; i < num_tones; i++)
                         freqs[i] = ThrobXToneFreqsNar[i];
                 bw = 47.0 / THROB_SAMPLE_RATE;
@@ -185,21 +183,22 @@ throb::throb(trx_mode throb_mode) : modem()
                 fp = mk_semi_pulse(symlen / DOWN_SAMPLE);
                 num_tones = 11;
                 num_chars = 55;
-		idlesym = 0;
-		spacesym = 1;
+				idlesym = 0;
+				spacesym = 1;
                 for (int i = 0; i < num_tones; i++)
-			freqs[i] = ThrobXToneFreqsNar[i];
+					freqs[i] = ThrobXToneFreqsNar[i];
                 bw = 47.0 / THROB_SAMPLE_RATE;
                 break;
 
         case MODE_THROBX4: //NONSTANDARD
                 symlen = SYMLEN_4;
+				mk_full_pulse(symlen);
                 txpulse = mk_full_pulse(symlen);
                 fp = mk_full_pulse(symlen / DOWN_SAMPLE);
                 num_tones = 11;
                 num_chars = 55;
-		idlesym = 0;
-		spacesym = 1;
+				idlesym = 0;
+				spacesym = 1;
                 for (int i = 0; i < num_tones; i++)
                         freqs[i] = ThrobXToneFreqsWid[i];
                 bw = 94.0 / THROB_SAMPLE_RATE;
@@ -217,6 +216,7 @@ throb::throb(trx_mode throb_mode) : modem()
 
 	syncfilt = new C_FIR_filter();
 	syncfilt->init(symlen / DOWN_SAMPLE, 1, fp, NULL);
+	delete fp;
 	
 	snfilter = new Cmovavg(16);
 
@@ -230,7 +230,7 @@ throb::throb(trx_mode throb_mode) : modem()
 	syncpos = 0.5;
 
 	scope_data	= new double [SCOPE_DATA_LEN];
-	wfid = new id(this);
+//	wfid = new id(this);
 	
 	phaseacc = 0.0;
 	metric = 0.0;
@@ -486,9 +486,10 @@ void throb::sync(complex in)
 		dispbuf[0] = 0.0;
 		set_scope(dispbuf, 1);
 	}
+	dispbuf.next(); // change buffers
 }
 
-int throb::rx_process(double *buf, int len)
+int throb::rx_process(const double *buf, int len)
 {
 	complex z, *zp;
 	int i, n;
@@ -592,8 +593,8 @@ void throb::send(int symbol)
 		tone2 = (num_tones - 1) - tone2;
 	}
 
-	w1 = 2.0 * M_PI * (tx_frequency + freqs[tone1]) / THROB_SAMPLE_RATE;
-	w2 = 2.0 * M_PI * (tx_frequency + freqs[tone2]) / THROB_SAMPLE_RATE;
+	w1 = 2.0 * M_PI * (get_txfreq_woffset() + freqs[tone1]) / THROB_SAMPLE_RATE;
+	w2 = 2.0 * M_PI * (get_txfreq_woffset() + freqs[tone2]) / THROB_SAMPLE_RATE;
 
 	for (i = 0; i < symlen; i++)
 		outbuf[i] = txpulse[i] * 
@@ -615,6 +616,14 @@ int throb::tx_process()
 
 	c = get_tx_char();
 
+// end of transmission
+	if (c == 0x03 || stopflag) {
+		send(idlesym);
+		reset_syms(); //prepare RX. idle/space syms always start as 0 and 1, respectively.
+		cwid();
+		return -1;
+	}
+
 // TX buffer empty
 	if (c == -1) {
 		send(idlesym);	/* send idle throbs */
@@ -622,13 +631,6 @@ int throb::tx_process()
 		return 0;
 	}
 
-// end of transmission
-	if (c == 0x03 || stopflag) {
-		send(idlesym);
-		reset_syms(); //prepare RX. idle/space syms always start as 0 and 1, respectively.
-		return -1;
-	}
-
 	switch(mode) {
 	case MODE_THROB1:
 	case MODE_THROB2:
