@@ -22,19 +22,25 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
 #include <string>
 
 #include "trx.h"
 #include "main.h"
 #include "fl_digi.h"
 #include "ascii.h"
-#include "rigCAT.h"
+//#include "rigio.h"
+//#include "rigMEM.h"
 #include "misc.h"
 //#include "modeIO.h"
 #include "configuration.h"
 #include "macros.h"
 
-#include "FL/Fl.H"
+#include <FL/Fl.H>
+
+#include "mbuffer.h"
+#include "qrunner.h"
 
 using namespace std;
 
@@ -50,6 +56,7 @@ void	trx_tune_loop();
 
 
 Fl_Mutex	trx_mutex = PTHREAD_MUTEX_INITIALIZER;
+Fl_Mutex	trx_cond_mutex = PTHREAD_MUTEX_INITIALIZER;
 Fl_Cond		trx_cond = PTHREAD_COND_INITIALIZER;
 Fl_Thread	trx_thread;
 state_t 	trx_state;
@@ -62,7 +69,9 @@ cSound 		*scard;
 int			_trx_tune;
 unsigned char ucdata[SCBLOCKSIZE * 4];
 short int	*sidata;
-double		_trx_scdbl[SCBLOCKSIZE*2]; // double size of normal read to accept overruns
+// Double size of normal read to accept overruns; double buffered because it
+// is QUEUEd and used asynchronously by the GUI thread.
+mbuffer<double, SCBLOCKSIZE * 2, 2> _trx_scdbl;
 
 static int dummy = 0;
 static bool trxrunning = false;
@@ -82,22 +91,25 @@ void trx_trx_receive_loop()
 		try {
 			scard->Open(O_RDONLY, active_modem->get_samplerate());
 		}
-		catch (SndException e) {
+		catch (const SndException& e) {
+			put_status(e.what(), 1);
 			MilliSleep(10);
 			return;
 		}
 		active_modem->rx_init();
 
-		while (trx_state == STATE_RX) {
+		while (1) {
 			numread = scard->Read(_trx_scdbl, SCBLOCKSIZE);
-			if (numread != -1) {
-				if (trx_state == STATE_RX)
-					wf->sig_data(_trx_scdbl, numread);
-				if (trx_state == STATE_RX)
-					active_modem->rx_process(_trx_scdbl, numread);
+			if (numread == -1 || (trx_state != STATE_RX))
+				break;
+			if (numread > 0) {
+				QUEUE(CMP_CB(&waterfall::sig_data, wf, _trx_scdbl.c_array(), numread)); //wf->sig_data(_trx_scdbl, numread);
+				active_modem->rx_process(_trx_scdbl, numread);
+				_trx_scdbl.next(); // change buffers
 			}
 		}
-		scard->Close();
+		if (!scard->full_duplex())
+			scard->Close();
 	} else
 		MilliSleep(10);
 }
@@ -113,7 +125,8 @@ void trx_trx_transmit_loop()
 		try {
 			scard->Open(O_WRONLY, active_modem->get_samplerate());
 		}
-		catch (SndException e) {
+		catch (const SndException& e) {
+			put_status(e.what(), 1);
 			MilliSleep(10);
 			return;
 		}
@@ -125,12 +138,13 @@ void trx_trx_transmit_loop()
 			if (active_modem->tx_process() < 0)
 				trx_state = STATE_RX;
 		}
-		scard->Close();
+		if (!scard->full_duplex())
+			scard->Close();
 	} else
 		MilliSleep(10);
 
 	push2talk->set(false);
-	wf->set_XmtRcvBtn(false);
+	QUEUE_SYNC(CMP_CB(&waterfall::set_XmtRcvBtn, wf, false)); //wf->set_XmtRcvBtn(false);
 
 	if (progdefaults.useTimer == true) {
 		trx_start_macro_timer();
@@ -147,7 +161,8 @@ void trx_tune_loop()
 		try {
 			scard->Open(O_WRONLY, active_modem->get_samplerate());
 		}
-		catch (SndException e) {
+		catch (const SndException& e) {
+			put_status(e.what(), 1);
 			MilliSleep(10);
 			return;
 		}
@@ -157,24 +172,27 @@ void trx_tune_loop()
 
 		while (trx_state == STATE_TUNE) {
 			if (_trx_tune == 0) {
-				wf->set_XmtRcvBtn(true);
-				xmttune::keydown(active_modem->get_txfreq(), scard);
+				QUEUE_SYNC(CMP_CB(&waterfall::set_XmtRcvBtn, wf, true)); //wf->set_XmtRcvBtn(true);
+				xmttune::keydown(active_modem->get_txfreq_woffset(), scard);
 				_trx_tune = 1;
 			} else
-				xmttune::tune(active_modem->get_txfreq(), scard);
+				xmttune::tune(active_modem->get_txfreq_woffset(), scard);
 		}
-		scard->Close();
-		xmttune::keyup(active_modem->get_txfreq(), scard);
+		xmttune::keyup(active_modem->get_txfreq_woffset(), scard);
+		if (!scard->full_duplex())
+			scard->Close();
 		_trx_tune = 0;
 	} else
 		MilliSleep(10);
 
 	push2talk->set(false);
-	wf->set_XmtRcvBtn(false);
+	QUEUE_SYNC(CMP_CB(&waterfall::set_XmtRcvBtn, wf, false)); //wf->set_XmtRcvBtn(false);
 }
 
 void *trx_loop(void *args)
 {
+	SET_THREAD_ID(TRX_TID);
+
 	for (;;) {
 		if ( trx_state == STATE_ABORT) {
 			trx_state = STATE_ENDED;
@@ -209,12 +227,10 @@ void trx_start_modem_loop()
 		active_modem->shutdown();
 
 	active_modem = trx_m;
-	active_modem->set_freq(wf->carrier());
 	active_modem->init();
-
 	trx_state = STATE_RX;
-
-	wf->opmode();
+	signal_modem_ready();
+	QUEUE(CMP_CB(&waterfall::opmode, wf)); //wf->opmode();
 }
 
 void trx_start_modem(modem *m)
@@ -231,7 +247,14 @@ void trx_reset_loop()
 		delete scard;
 		scard = 0;
 	}
-	scard = new cSound(trx_scdev.c_str());
+#if USE_PORTAUDIO
+	if (progdefaults.btnAudioIOis == 1)
+		scard = new cSoundPA(trx_scdev.c_str());
+	else
+		scard = new cSoundOSS(trx_scdev.c_str());
+#else
+	scard = new cSoundOSS(trx_scdev.c_str());
+#endif
 	trx_state = STATE_RX;	
 }
 
@@ -250,15 +273,15 @@ static void macro_timer(void *)
 	if (progdefaults.timeout == 0) {
 		progdefaults.useTimer = false;
 		macros.execute(progdefaults.macronumber);
-		Fl::lock();
+		FL_LOCK();
 		btnMacroTimer->hide();
-		Fl::unlock();
+		FL_UNLOCK();
 	} else {
 		sprintf(timermsg,"Timer: %d", progdefaults.timeout);
-		Fl::lock();
+		FL_LOCK();
 		btnMacroTimer->label(timermsg);
 		btnMacroTimer->redraw_label();
-		Fl::unlock();
+		FL_UNLOCK();
 		Fl::repeat_timeout(1.0, macro_timer);
 	}
 }
@@ -267,11 +290,11 @@ void trx_start_macro_timer()
 {
 	Fl::add_timeout(1.0, macro_timer);
 	sprintf(timermsg,"Timer: %d", progdefaults.timeout);
-	Fl::lock();
+	FL_LOCK();
 	btnMacroTimer->label(timermsg);
 	btnMacroTimer->redraw_label();
 	btnMacroTimer->show();
-	Fl::unlock();
+	FL_UNLOCK();
 }
 
 void trx_start(const char *scdev)
@@ -282,7 +305,14 @@ void trx_start(const char *scdev)
 	}
 	
 	if (scard) delete scard;
-	scard = new cSound(scdev);
+#if USE_PORTAUDIO
+	if (progdefaults.btnAudioIOis == 1)
+		scard = new cSoundPA(scdev);
+	else
+		scard = new cSoundOSS(scdev);
+#else
+	scard = new cSoundOSS(scdev);
+#endif
 
 	trx_state = STATE_RX;
 	_trx_tune = 0;
@@ -308,5 +338,37 @@ void trx_close() {
 
 //---------------------------------------------------------------------
 
+void wait_modem_ready_prep(void)
+{
+#ifndef NDEBUG
+        if (GET_THREAD_ID() == TRX_TID)
+                cerr << "trx thread called wait_modem_ready_prep!\n";
+#endif
+
+        fl_lock(&trx_cond_mutex);
+}
+
+void wait_modem_ready_cmpl(void)
+{
+#ifndef NDEBUG
+        if (GET_THREAD_ID() == TRX_TID)
+                cerr << "trx thread called wait_modem_ready_cmpl!\n";
+#endif
+
+        fl_cond_wait(&trx_cond, &trx_cond_mutex);
+        fl_unlock(&trx_cond_mutex);
+}
 
 
+void signal_modem_ready(void)
+{
+#ifndef NDEBUG
+        if (GET_THREAD_ID() != TRX_TID)
+                cerr << "thread " << GET_THREAD_ID()
+                     << " called signal_modem_ready!\n";
+#endif
+
+        fl_lock(&trx_cond_mutex);
+        fl_cond_bcast(&trx_cond);
+        fl_unlock(&trx_cond_mutex);
+}
