@@ -21,12 +21,15 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
+#include <sys/stat.h>
+#include <sys/types.h>
+
 #include <FL/fl_ask.H>
 
 #include "picture.h"
 
-#include <iostream>
-
 picture::picture (int X, int Y, int W, int H) :
 	Fl_Widget (X, Y, W, H) 
 {
@@ -49,19 +52,19 @@ picture::~picture()
 void picture::video(unsigned char *data, int len )
 {
 	if (len > bufsize) return;
-	Fl::lock();
+	FL_LOCK_D();
 	memmove( vidbuf, data, len );
 	redraw();
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 void picture::pixel(unsigned char data, int pos)
 {
 	if (pos < 0 || pos >= bufsize) return;
-	Fl::lock();
+	FL_LOCK_D();
 	vidbuf[pos] = data;
 	redraw();
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 unsigned char picture::pixel(int pos)
@@ -72,16 +75,16 @@ unsigned char picture::pixel(int pos)
 
 void picture::clear()
 {
-	Fl::lock();
+	FL_LOCK_D();
 	memset(vidbuf, 0, bufsize);
 	redraw();
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 
 void picture::resize(int x, int y, int w, int h)
 {
-	Fl::lock();
+	FL_LOCK_D();
 	width = w;
 	height = h;
 	delete [] vidbuf;
@@ -89,7 +92,7 @@ void picture::resize(int x, int y, int w, int h)
 	vidbuf = new unsigned char[bufsize];
 	memset( vidbuf, 0, bufsize );
 	Fl_Widget::resize(x,y,w,h);
-	Fl::unlock();
+	FL_UNLOCK_D();
 }
 
 void picture::draw()
@@ -206,16 +209,36 @@ int picture::handle(int event)
 // save_jpeg - Write a captured picture to a JPEG format file.
 //
 
-int picture::save_jpeg(char *filename)
+int picture::save_jpeg(const char *filename)
 {
 	unsigned char		*ptr;		// Pointer to image data
-	FILE				*fp;		// File pointer
+	FILE				*fp = 0;	// File pointer
 	struct jpeg_compress_struct	info;		// Compressor info
 	struct jpeg_error_mgr		err;		// Error handler info
 
-// Create the output file...
-	if ((fp = fopen(filename, "wb")) == NULL)
-		return -1;
+	size_t flen = strlen(filename);
+	if (filename[flen - 1] == '/') {
+		// if the filename ends in a slash we will generate
+		// a timestamped filename in the following  format:
+		const char t[] = "pic_YYYY-MM-DD_HH:MM:SSz.jpg";
+
+		size_t newlen = flen + sizeof(t);
+		char newfn[newlen];
+		memcpy(newfn, filename, flen);
+
+		time_t time_sec = time(0);
+		struct tm ztime;
+		(void)gmtime_r(&time_sec, &ztime);
+
+		if (strftime(newfn + flen, newlen - flen, "pic_%F_%Tz.jpg", &ztime) > 0) {
+			mkdir(filename, 0777);
+			if ((fp = fopen(newfn, "wb")) == NULL)
+				return -1;
+		}
+	}
+	else
+		if ((fp = fopen(filename, "wb")) == NULL)
+			return -1;
 
 // Setup the JPEG compression stuff...
 	info.err = jpeg_std_error(&err);
