@@ -21,6 +21,8 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 // ----------------------------------------------------------------------------
 
+#include <config.h>
+
 #include "analysis.h"
 #include "waterfall.h"
 #include "Config.h"
@@ -37,13 +39,13 @@ void anal::tx_init(cSound *sc)
 void anal::rx_init()
 {
 	phaseacc = 0;
+	put_MODEstatus(mode);
 }
 
 void anal::init()
 {
 	modem::init();
 	rx_init();
-	clear_StatusMessages();
 	digiscope->mode(Digiscope::RTTY);
 }
 
@@ -73,8 +75,6 @@ void anal::restart()
 	
 	sum = 0.0;
 	fout_1 = fout_2 = 0.0;
-	
-	put_MODEstatus(mode);
 }
 
 anal::anal()
@@ -98,10 +98,7 @@ anal::anal()
 
 void anal::clear_syncscope()
 {
-	double *data = new double[symbollen];
-	for (int i = 0; i < symbollen; data[i++] = 0.0);
-	set_scope(data, symbollen, false);
-	delete [] data;
+	set_scope(0, 0, false);
 }
 
 complex anal::mixer(complex in)
@@ -127,7 +124,7 @@ void anal::writeFile()
 	analysisFile.close();
 }
 
-int anal::rx_process(double *buf, int len)
+int anal::rx_process(const double *buf, int len)
 {
 	complex z, *zp;
 	double fin;
@@ -156,6 +153,7 @@ int anal::rx_process(double *buf, int len)
 			dspcnt--;
 			if (dspcnt == 0) {
 				set_scope(pipe, symbollen, false);
+				pipe.next(); // change buffers
 // filter using second moving average filter & display the result
 				fout_2 = favg->run(fout_1);
 				if (wf->USB())
