@@ -24,6 +24,8 @@
 // This modem is only used for reception of WWV "tick" signals to determine
 // the correction factor to be applied to the sound card oscillator.
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <iostream>
@@ -63,7 +65,6 @@ void wwv::init()
 wwv::~wwv() {
 	if (hilbert) delete hilbert;
 	if (vidfilter) delete vidfilter;
-	if (buffer) delete [] buffer;
 }
 
 
@@ -75,7 +76,6 @@ wwv::wwv() : modem()
 	bandwidth = 200;
 	samplerate = 8000;	
 
-	buffer = new double[1000];
 // phase increment expected at the tick freq 
 	phaseincr = 2.0 * M_PI * frequency / samplerate;
 
@@ -112,6 +112,7 @@ void wwv::update_syncscope()
 		set_video(&buffer[400], 200);
 	else
 		set_video(buffer, 1000);
+	buffer.next(); // change buffers
 }
 
 
@@ -121,7 +122,7 @@ void wwv::update_syncscope()
 // Nominal sound card sampling rate is set to 8000 Hz
 //=======================================================================
 
-int wwv::rx_process(double *buf, int len)
+int wwv::rx_process(const double *buf, int len)
 {
 	complex z, znco;
 
